@@ -1,32 +1,38 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: asapinstance.c 1730 2008-02-14 10:53:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
+ *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and
+ * University of Essen, Institute of Computer Networking Technology.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "tdtypes.h"
@@ -268,8 +274,6 @@ static void asapInstanceConfigure(struct ASAPInstance* asapInstance,
                                                                               ASAP_DEFAULT_REGISTRAR_REQUEST_TIMEOUT);
    asapInstance->RegistrarResponseTimeout = (unsigned long long)tagListGetData(tags, TAG_RspLib_RegistrarResponseTimeout,
                                                                                ASAP_DEFAULT_REGISTRAR_RESPONSE_TIMEOUT);
-   asapInstance->CacheElementTimeout = (unsigned long long)tagListGetData(tags, TAG_RspLib_CacheElementTimeout,
-                                                                          ASAP_DEFAULT_CACHE_ELEMENT_TIMEOUT);
 
 
    /* ====== Show results =================================================== */
@@ -278,7 +282,6 @@ static void asapInstanceConfigure(struct ASAPInstance* asapInstance,
    fprintf(stdlog, "registrar.request.maxtrials   = %u\n",     (unsigned int)asapInstance->RegistrarRequestMaxTrials);
    fprintf(stdlog, "registrar.request.timeout     = %lluus\n", asapInstance->RegistrarRequestTimeout);
    fprintf(stdlog, "registrar.response.timeout    = %lluus\n", asapInstance->RegistrarResponseTimeout);
-   fprintf(stdlog, "cache.elementtimeout          = %lluus\n", asapInstance->CacheElementTimeout);
    LOG_END
 }
 
@@ -287,8 +290,8 @@ static void asapInstanceConfigure(struct ASAPInstance* asapInstance,
 static bool asapInstanceConnectToRegistrar(struct ASAPInstance* asapInstance,
                                            int                  sd)
 {
-#ifdef SCTP_DELAYED_ACK_TIME
-   struct sctp_assoc_value sctpAssocValue;
+#ifdef sctp_sack_info
+   struct sctp_sack_info sctpSACKInfo;
 #endif
 
    if(asapInstance->RegistrarSocket < 0) {
@@ -326,19 +329,20 @@ static bool asapInstanceConnectToRegistrar(struct ASAPInstance* asapInstance,
       fprintf(stdlog, "Connected to registrar $%08x\n", asapInstance->RegistrarIdentifier);
       LOG_END
 
-#ifdef SCTP_DELAYED_ACK_TIME
+#ifdef sctp_sack_info
       /* ====== Tune SACK handling ======================================= */
-      sctpAssocValue.assoc_id    = 0;
-      sctpAssocValue.assoc_value = 0;
+      sctpSACKInfo.sack_assoc_id = 0;
+      sctpSACKInfo.sack_delay    = 0;
+      sctpSACKInfo.sack_freq     = 1;
       if(ext_setsockopt(asapInstance->RegistrarSocket,
-                        IPPROTO_SCTP, SCTP_DELAYED_ACK_TIME,
-                        &sctpAssocValue, sizeof(sctpAssocValue)) < 0) {
+                        IPPROTO_SCTP, SCTP_DELAYED_SACK,
+                        &sctpSACKInfo, sizeof(sctpSACKInfo)) < 0) {
          LOG_WARNING
-         logerror("Unable to set SCTP_DELAYED_ACK_TIME");
+         logerror("Unable to set SCTP_DELAYED_SACK");
          LOG_END
       }
 #else
-#warning SCTP_DELAYED_ACK_TIME is not defined - Unable to tune SACK handling for DPF policies!
+#warning SCTP_DELAYED_SACK/sctp_sack_info is not supported - Unable to tune SACK handling!
 #endif
    }
    return(true);
@@ -664,10 +668,89 @@ unsigned int asapInstanceDeregister(struct ASAPInstance*            asapInstance
 }
 
 
+/* ###### Do name lookup from cache ###################################### */
+static unsigned int asapInstanceHandleResolutionFromCache(
+                       struct ASAPInstance*               asapInstance,
+                       struct PoolHandle*                 poolHandle,
+                       void**                             nodePtrArray,
+                       struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
+                       size_t*                            poolElementNodes,
+                       unsigned int                       (*convertFunction)(const struct ST_CLASS(PoolElementNode)* poolElementNode,
+                                                                             void*                                   ptr),
+                       const bool                         purgeOutOfDateElements)
+{
+   unsigned int result;
+   size_t       i;
+
+   dispatcherLock(asapInstance->StateMachine);
+
+   LOG_VERBOSE
+   fprintf(stdlog, "Handle Resolution for pool ");
+   poolHandlePrint(poolHandle, stdlog);
+   fputs(":\n", stdlog);
+   ST_CLASS(poolHandlespaceManagementPrint)(&asapInstance->Cache,
+                                            stdlog, PENPO_ONLY_POLICY);
+   LOG_END
+
+   if(purgeOutOfDateElements) {
+      i = ST_CLASS(poolHandlespaceManagementPurgeExpiredPoolElements)(
+            &asapInstance->Cache,
+            getMicroTime());
+      LOG_VERBOSE
+      fprintf(stdlog, "Purged %u out-of-date elements\n", (unsigned int)i);
+      LOG_END
+   }
+
+   if(ST_CLASS(poolHandlespaceManagementHandleResolution)(
+         &asapInstance->Cache,
+         poolHandle,
+         poolElementNodeArray,
+         poolElementNodes,
+         *poolElementNodes,
+         1000000000) == RSPERR_OKAY) {
+
+      LOG_VERBOSE
+      fprintf(stdlog, "Got %u items:\n", (unsigned int)*poolElementNodes);
+      for(i = 0;i < *poolElementNodes;i++) {
+         fprintf(stdlog, "#%u: ", (unsigned int)i + 1);
+         ST_CLASS(poolElementNodePrint)(poolElementNodeArray[i],
+                                        stdlog, PENPO_FULL);
+      }
+      fputs("\n", stdlog);
+      LOG_END
+
+      result = RSPERR_OKAY;
+      for(i = 0;i < *poolElementNodes;i++) {
+          if(convertFunction(poolElementNodeArray[i], &nodePtrArray[i]) != 0) {
+             result = RSPERR_OUT_OF_MEMORY;
+          }
+      }
+      if(result != RSPERR_OKAY) {
+         for(i = 0;i < *poolElementNodes;i++) {
+            free(nodePtrArray[i]);
+            nodePtrArray[i] = 0;
+         }
+         *poolElementNodes = 0;
+      }
+   }
+   else {
+      result = RSPERR_NOT_FOUND;
+   }
+
+   dispatcherUnlock(asapInstance->StateMachine);
+   return(result);
+}
+
+
 /* ###### Do name lookup ################################################# */
-static unsigned int asapInstanceHandleResolutionAtRegistrar(struct ASAPInstance* asapInstance,
-                                                            struct PoolHandle*   poolHandle,
-                                                            const size_t         items)
+static unsigned int asapInstanceHandleResolutionAtRegistrar(struct ASAPInstance*               asapInstance,
+                                                            struct PoolHandle*                 poolHandle,
+                                                            void**                             nodePtrArray,
+                                                            struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
+                                                            size_t*                            poolElementNodes,
+                                                            unsigned int                       (*convertFunction)(const struct ST_CLASS(PoolElementNode)* poolElementNode,
+                                                                                                                  void*                                   ptr),
+                                                            const unsigned long long           cacheElementTimeout)
 {
    struct ST_CLASS(PoolElementNode)* newPoolElementNode;
    struct RSerPoolMessage*           message;
@@ -680,7 +763,7 @@ static unsigned int asapInstanceHandleResolutionAtRegistrar(struct ASAPInstance*
       message->Type      = AHT_HANDLE_RESOLUTION;
       message->Flags     = 0x00;
       message->Handle    = *poolHandle;
-      message->Addresses = ((items != RSPGETADDRS_MAX) && (asapInstance->CacheElementTimeout > 0)) ? 0 : items;
+      message->Addresses = ((*poolElementNodes != RSPGETADDRS_MAX) && (cacheElementTimeout > 0)) ? 0 : *poolElementNodes;
 
       result = asapInstanceDoIO(asapInstance, message, &response);
       if(result == RSPERR_OKAY) {
@@ -690,8 +773,9 @@ static unsigned int asapInstanceHandleResolutionAtRegistrar(struct ASAPInstance*
                     (unsigned int)response->PoolElementPtrArraySize);
             LOG_END
 
-            /* ====== Propagate results into PU-side cache =============== */
             dispatcherLock(asapInstance->StateMachine);
+
+            /* ====== Propagate results into PU-side cache =============== */
             for(i = 0;i < response->PoolElementPtrArraySize;i++) {
                LOG_VERBOSE2
                fputs("Adding pool element to cache: ", stdlog);
@@ -722,8 +806,16 @@ static unsigned int asapInstanceHandleResolutionAtRegistrar(struct ASAPInstance*
                ST_CLASS(poolHandlespaceManagementRestartPoolElementExpiryTimer)(
                   &asapInstance->Cache,
                   newPoolElementNode,
-                  asapInstance->CacheElementTimeout);
+                  cacheElementTimeout);
             }
+
+            /* ====== Select PEs from cache ============================== */
+            result = asapInstanceHandleResolutionFromCache(
+                        asapInstance, poolHandle,
+                        nodePtrArray,
+                        poolElementNodeArray,
+                        poolElementNodes, convertFunction, false);
+
             dispatcherUnlock(asapInstance->StateMachine);
             /* =========================================================== */
 
@@ -759,82 +851,31 @@ static unsigned int asapInstanceHandleResolutionAtRegistrar(struct ASAPInstance*
 }
 
 
-/* ###### Do name lookup from cache ###################################### */
-static unsigned int asapInstanceHandleResolutionFromCache(
-                       struct ASAPInstance*               asapInstance,
-                       struct PoolHandle*                 poolHandle,
-                       struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
-                       size_t*                            poolElementNodes,
-                       const bool                         purgeOutOfDateElements)
-{
-   unsigned int result;
-   size_t       i;
-
-   dispatcherLock(asapInstance->StateMachine);
-
-   LOG_VERBOSE
-   fprintf(stdlog, "Handle Resolution for pool ");
-   poolHandlePrint(poolHandle, stdlog);
-   fputs(":\n", stdlog);
-   ST_CLASS(poolHandlespaceManagementPrint)(&asapInstance->Cache,
-                                            stdlog, PENPO_ONLY_POLICY);
-   LOG_END
-
-   if(purgeOutOfDateElements) {
-      i = ST_CLASS(poolHandlespaceManagementPurgeExpiredPoolElements)(
-            &asapInstance->Cache,
-            getMicroTime());
-      LOG_VERBOSE
-      fprintf(stdlog, "Purged %u out-of-date elements\n", (unsigned int)i);
-      LOG_END
-   }
-
-   if(ST_CLASS(poolHandlespaceManagementHandleResolution)(
-         &asapInstance->Cache,
-         poolHandle,
-         poolElementNodeArray,
-         poolElementNodes,
-         *poolElementNodes,
-         1000000000) == RSPERR_OKAY) {
-      LOG_VERBOSE
-      fprintf(stdlog, "Got %u items:\n", (unsigned int)*poolElementNodes);
-      for(i = 0;i < *poolElementNodes;i++) {
-         fprintf(stdlog, "#%u: ", (unsigned int)i + 1);
-         ST_CLASS(poolElementNodePrint)(poolElementNodeArray[i],
-                                        stdlog, PENPO_FULL);
-      }
-      fputs("\n", stdlog);
-      LOG_END
-      result = RSPERR_OKAY;
-   }
-   else {
-      result = RSPERR_NOT_FOUND;
-   }
-
-   dispatcherUnlock(asapInstance->StateMachine);
-   return(result);
-}
-
-
 /* ###### Do handle resolution for given pool handle ##################### */
+#define HRES_POOL_ELEMENT_NODE_ARRAY_SIZE 1024
 unsigned int asapInstanceHandleResolution(
-                struct ASAPInstance*               asapInstance,
-                struct PoolHandle*                 poolHandle,
-                struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
-                size_t*                            poolElementNodes)
+                struct ASAPInstance*     asapInstance,
+                struct PoolHandle*       poolHandle,
+                void**                   nodePtrArray,
+                size_t*                  nodePtrs,
+                unsigned int             (*convertFunction)(const struct ST_CLASS(PoolElementNode)* poolElementNode,
+                                                            void*                                   ptr),
+                const unsigned long long cacheElementTimeout)
 {
-   unsigned int result;
-   const size_t originalPoolElementNodes = *poolElementNodes;
+   struct ST_CLASS(PoolElementNode)* poolElementNodeArray[HRES_POOL_ELEMENT_NODE_ARRAY_SIZE];
+   const size_t                      originalPoolElementNodes = min(HRES_POOL_ELEMENT_NODE_ARRAY_SIZE, *nodePtrs);
+   unsigned int                      result;
 
    LOG_VERBOSE
    fputs("Trying handle resolution from cache...\n", stdlog);
    LOG_END
 
+   *nodePtrs = originalPoolElementNodes;
    result = asapInstanceHandleResolutionFromCache(
                asapInstance, poolHandle,
-               poolElementNodeArray,
-               poolElementNodes,
-               true);
+               nodePtrArray,
+               (struct ST_CLASS(PoolElementNode)**)&poolElementNodeArray,
+               nodePtrs, convertFunction, true);
    if(result != RSPERR_OKAY) {
       LOG_VERBOSE
       fputs("No results in cache. Trying handle resolution at registrar...\n", stdlog);
@@ -842,17 +883,15 @@ unsigned int asapInstanceHandleResolution(
 
       /* The amount is now 0 (since handle resolution was not successful).
          Set it to its original value. */
-      *poolElementNodes = originalPoolElementNodes;
+      *nodePtrs = originalPoolElementNodes;
 
-      result = asapInstanceHandleResolutionAtRegistrar(asapInstance, poolHandle, *poolElementNodes);
-      if(result == RSPERR_OKAY) {
-         result = asapInstanceHandleResolutionFromCache(
-                     asapInstance, poolHandle,
-                     poolElementNodeArray,
-                     poolElementNodes,
-                     false);
-      }
-      else {
+      result = asapInstanceHandleResolutionAtRegistrar(
+                  asapInstance, poolHandle,
+                  nodePtrArray,
+                  (struct ST_CLASS(PoolElementNode)**)&poolElementNodeArray,
+                  nodePtrs, convertFunction,
+                  cacheElementTimeout);
+      if(result != RSPERR_OKAY) {
          LOG_VERBOSE
          fputs("Handle resolution not succesful\n", stdlog);
          LOG_END
@@ -1006,7 +1045,7 @@ static void asapInstanceHandleEndpointKeepAlive(
          LOG_END
          result = rserpoolMessageSend(IPPROTO_SCTP,
                                     asapInstance->RegistrarSocket,
-                                    0, 0, 0,
+                                    0, 0, 0, 0,
                                     message);
       }
    }
@@ -1025,7 +1064,7 @@ static void asapInstanceHandleEndpointKeepAlive(
             LOG_END
             result = rserpoolMessageSend(IPPROTO_SCTP,
                                          asapInstance->RegistrarSocket,
-                                         0, 0, 0,
+                                         0, 0, 0, 0,
                                          message);
 
             poolElementNode = ST_CLASS(poolNodeGetNextPoolElementNodeFromIndex)(poolNode, poolElementNode);
@@ -1260,7 +1299,7 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
    struct ASAPInterThreadMessage* nextAITM;
    unsigned int                   result;
 
-   asapInstanceConnectToRegistrar(asapInstance, -1);
+   /* ====== Get next AITM =============================================== */
    interThreadMessagePortLock(&asapInstance->MainLoopPort);
    if(asapInstance->LastAITM) {
       aitm = (struct ASAPInterThreadMessage*)interThreadMessagePortGetNextMessage(
@@ -1270,6 +1309,16 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
    else {
       aitm = (struct ASAPInterThreadMessage*)interThreadMessagePortGetFirstMessage(&asapInstance->MainLoopPort);
    }
+
+   /* ====== If there is an AITM, connect to registrar if necessary ====== */
+   if( (asapInstance->RegistrarSocket < 0) &&
+       ( (aitm) || ((ST_CLASS(poolHandlespaceManagementGetPoolElements)(&asapInstance->OwnPoolElements) > 0)) ) ) {
+      interThreadMessagePortUnlock(&asapInstance->MainLoopPort);
+      asapInstanceConnectToRegistrar(asapInstance, -1);
+      interThreadMessagePortLock(&asapInstance->MainLoopPort);
+   }
+
+   /* ====== Handle AITMs ================================================ */
    while(aitm != NULL) {
       nextAITM = (struct ASAPInterThreadMessage*)interThreadMessagePortGetNextMessage(&asapInstance->MainLoopPort, &aitm->Node);
       aitm->TransmissionTrials++;
@@ -1302,7 +1351,7 @@ static void asapInstanceHandleQueuedAITMs(struct ASAPInstance* asapInstance)
             LOG_END
             result = rserpoolMessageSend(IPPROTO_SCTP,
                                          asapInstance->RegistrarSocket,
-                                         0, 0,
+                                         0, 0, 0,
                                          asapInstance->RegistrarRequestTimeout,
                                          aitm->Request);
             if(result == false) {
