@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: calcappclient.cc 1726 2008-02-14 09:56:55Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: calcappclient.cc 1352 2007-04-20 12:55:53Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "rserpool.h"
@@ -30,11 +34,14 @@
 #include "netutilities.h"
 #include "breakdetector.h"
 #include "randomizer.h"
+#include "timeutilities.h"
+#include "loglevel.h"
 #include "debug.h"
 #ifdef ENABLE_CSP
 #include "componentstatuspackets.h"
 #endif
 
+#include <math.h>
 #include <ext_socket.h>
 #include <iostream>
 
@@ -215,7 +222,7 @@ void sendCalcAppRequest(struct Process* process)
 
    ssize_t result = rsp_sendmsg(process->RSerPoolSocketDescriptor,
                                 (void*)&message, sizeof(message), 0,
-                                0, htonl(PPID_CALCAPP), 0, 0,
+                                0, htonl(PPID_CALCAPP), 0, 0, 0,
                                 0);
    if(result <= 0) {
       cerr << "WARNING: Unable to send CalcAppRequest" << endl;
@@ -238,7 +245,7 @@ void sendCalcAppKeepAlive(struct Process* process)
 
    ssize_t result = rsp_sendmsg(process->RSerPoolSocketDescriptor,
                                 (void*)&message, sizeof(message), 0,
-                                0, htonl(PPID_CALCAPP), 0, 0,
+                                0, htonl(PPID_CALCAPP), 0, 0, 0,
                                 0);
    if(result <= 0) {
       cerr << "WARNING: Unable to send CalcAppKeepAlive" << endl;
@@ -260,7 +267,7 @@ void sendCalcAppKeepAliveAck(struct Process* process)
 
    ssize_t result = rsp_sendmsg(process->RSerPoolSocketDescriptor,
                                 (void*)&message, sizeof(message), 0,
-                                0, htonl(PPID_CALCAPP), 0, 0,
+                                0, htonl(PPID_CALCAPP), 0, 0, 0,
                                 0);
    if(result <= 0) {
       cerr << "WARNING: Unable to send CalcAppKeepAliveAck" << endl;
@@ -307,7 +314,7 @@ void handleCalcAppReject(struct Process* process,
    process->Status                         = PS_Failover;
    process->KeepAliveTimeoutTimeStamp      = ~0ULL;
    process->KeepAliveTransmissionTimeStamp = ~0ULL;
-   rsp_forcefailover(process->RSerPoolSocketDescriptor);
+   rsp_forcefailover(process->RSerPoolSocketDescriptor, FFF_NONE, 0);
 
    /* No cookie for failover is available, therefore it is necessary
       to restart! */
@@ -470,7 +477,7 @@ bool handleKeepAliveTimeoutTimer(struct Process* process)
 {
    cout << "Timeout -> Failover" << endl;
 
-   rsp_forcefailover(process->RSerPoolSocketDescriptor);
+   rsp_forcefailover(process->RSerPoolSocketDescriptor, FFF_UNREACHABLE, 0);
 
    process->Status                         = PS_Processing;
    process->KeepAliveTimeoutTimeStamp      = ~0ULL;
@@ -519,7 +526,7 @@ void handleEvents(Process*    process,
          if((notification->rn_header.rn_type == RSERPOOL_FAILOVER) &&
             (notification->rn_failover.rf_state == RSERPOOL_FAILOVER_NECESSARY)) {
             puts("--- FAILOVER ---");
-            rsp_forcefailover(process->RSerPoolSocketDescriptor);
+            rsp_forcefailover(process->RSerPoolSocketDescriptor, FFF_NONE, 0);
             /* No cookie for failover is available, therefore it is necessary
                to restart! */
             if(!rsp_has_cookie(process->RSerPoolSocketDescriptor)) {
@@ -619,7 +626,7 @@ void runProcess(const char* poolHandle, const char* objectName, unsigned long lo
       process.RSerPoolSocketDescriptor = rsp_socket(0, SOCK_SEQPACKET, IPPROTO_SCTP);
       if(process.RSerPoolSocketDescriptor >= 0) {
          if(rsp_connect(process.RSerPoolSocketDescriptor,
-                        (unsigned char*)poolHandle, strlen(poolHandle)) == 0) {
+                        (unsigned char*)poolHandle, strlen(poolHandle), 0) == 0) {
             sendCalcAppRequest(&process);
 
             while((process.Status != PS_Failed) &&
@@ -742,10 +749,10 @@ finished:
 int main(int argc, char** argv)
 {
    struct rsp_info    info;
-   char*              poolHandle        = "CalcAppPool";
-   char*              vectorFileName    = "CalcAppPU.vec";
-   char*              scalarFileName    = "CalcAppPU.sca";
-   char*              objectName        = "scenario.calcapppooluser[0]";
+   const char*        poolHandle        = "CalcAppPool";
+   const char*        vectorFileName    = "CalcAppPU.vec";
+   const char*        scalarFileName    = "CalcAppPU.sca";
+   const char*        objectName        = "scenario.calcapppooluser[0]";
    unsigned long long startupDelayStamp = getMicroTime();
    int                i;
 
@@ -757,7 +764,7 @@ int main(int argc, char** argv)
       if(rsp_initarg(&info, argv[i])) {
          /* rsplib argument */
       }
-      if(!(strncmp(argv[i],"-poolhandle=",12))) {
+      else if(!(strncmp(argv[i],"-poolhandle=",12))) {
          poolHandle = (char*)&argv[i][12];
       }
       else if(!(strncmp(argv[i], "-object=" ,8))) {
