@@ -1,28 +1,33 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: calcappservice.cc 1726 2008-02-14 09:56:55Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: calcappservice.cc 1391 2007-05-23 07:27:45Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
+#include <math.h>
 #include <iostream>
 
 #include "calcappservice.h"
@@ -379,7 +384,7 @@ void CalcAppServer::sendCalcAppAccept(CalcAppServer::CalcAppServerJob* job)
    message.JobID  = htonl(job->JobID);
    if(rsp_sendmsg(RSerPoolSocketDescriptor,
                   (void*)&message, sizeof(message), 0,
-                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0) < 0) {
+                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0, 0) < 0) {
       logerror("Unable to send CalcAppAccept");
       removeJob(job);
    }
@@ -398,7 +403,7 @@ void CalcAppServer::sendCalcAppReject(rserpool_session_t sessionID, uint32_t job
    message.JobID  = htonl(jobID);
    if(rsp_sendmsg(RSerPoolSocketDescriptor,
                   (void*)&message, sizeof(message), 0,
-                  sessionID, htonl(PPID_CALCAPP), 0, 0, 0) < 0) {
+                  sessionID, htonl(PPID_CALCAPP), 0, 0, 0, 0) < 0) {
       logerror("Unable to send CalcAppReject");
    }
 }
@@ -428,7 +433,7 @@ void CalcAppServer::sendCalcAppComplete(CalcAppServer::CalcAppServerJob* job)
    message.Completed = hton64((unsigned long long)rint(job->Completed));
    if(rsp_sendmsg(RSerPoolSocketDescriptor,
                   (void*)&message, sizeof(message), 0,
-                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0) < 0) {
+                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0, 0) < 0) {
       logerror("Unable to send CalcAppAccept");
    }
 
@@ -481,7 +486,7 @@ void CalcAppServer::sendCalcAppKeepAliveAck(CalcAppServer::CalcAppServerJob* job
    message.JobID  = htonl(job->JobID);
    if(rsp_sendmsg(RSerPoolSocketDescriptor,
                   (void*)&message, sizeof(message), 0,
-                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0) < 0) {
+                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0, 0) < 0) {
       logerror("Unable to send CalcAppKeepAlive");
       removeJob(job);
       return;
@@ -501,7 +506,7 @@ void CalcAppServer::sendCalcAppKeepAlive(CalcAppServer::CalcAppServerJob* job)
    message.JobID  = htonl(job->JobID);
    if(rsp_sendmsg(RSerPoolSocketDescriptor,
                   (void*)&message, sizeof(message), 0,
-                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0) < 0) {
+                  job->SessionID, htonl(PPID_CALCAPP), 0, 0, 0, 0) < 0) {
       logerror("Unable to send CalcAppKeepAlive");
       removeJob(job);
       return;
