@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: cspmonitor.c 1726 2008-02-14 09:56:55Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: cspmonitor.c 1335 2007-02-13 12:32:36Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "tdtypes.h"
@@ -31,6 +35,10 @@
 #include "componentstatusreporter.h"
 #include "simpleredblacktree.h"
 
+#include <stdlib.h>
+#include <unistd.h>
+#include <math.h>
+#include <string.h>
 #include <sys/poll.h>
 #include <ext_socket.h>
 
@@ -278,7 +286,11 @@ int main(int argc, char** argv)
    struct SimpleRedBlackTree objectStorage;
    union sockaddr_union      localAddress;
    struct pollfd             ufds;
+   unsigned long long        now;
    unsigned long long        lastUpdate;
+   unsigned long long        updateInterval = 1000000;
+   size_t                    lastElements   = 0;
+   size_t                    elements;
    int                       result;
    int                       reuse;
    int                       sd;
@@ -299,6 +311,12 @@ int main(int argc, char** argv)
             exit(1);
          }
       }
+      else if(!(strncmp(argv[n], "-updateinterval=", 16))) {
+         updateInterval = 1000 * atol((char*)&argv[n][16]);
+         if(updateInterval < 100000) {
+            updateInterval = 100000;
+         }
+      }
       else if(!(strcmp(argv[n], "-compact"))) {
          useCompactMode = true;
       }
@@ -307,6 +325,7 @@ int main(int argc, char** argv)
       }
       else {
          printf("Bad argument \"%s\"!\n" ,argv[n]);
+         fprintf(stderr, "Usage: %s {-localaddress=address:port} {-updateinterval=milliseconds} {-compact|-full}\n", argv[0]);
          exit(1);
       }
    }
@@ -337,23 +356,32 @@ int main(int argc, char** argv)
    printf("\x1b[;H\x1b[2J");
 
    while(!breakDetected()) {
-      ufds.fd     = sd;
-      ufds.events = POLLIN;
-      lastUpdate = getMicroTime();
-      while(getMicroTime() - lastUpdate < 350000) {
-         result = ext_poll(&ufds, 1, 175);
+      ufds.fd          = sd;
+      ufds.events      = POLLIN;
+      now = lastUpdate = getMicroTime();
+      while(now - lastUpdate < updateInterval) {
+         now = getMicroTime();
+         result = ext_poll(&ufds, 1,
+                           ((lastUpdate + updateInterval) > now) ?
+                              (int)((lastUpdate + updateInterval - now) / 1000) : 0);
          if((result > 0) && (ufds.revents & POLLIN)) {
             handleMessage(sd, &objectStorage);
          }
       }
       purgeCSPObjects(&objectStorage);
 
-      printf("\x1b[;H");
-      printTimeStamp(stdout);
-      puts("Current Component Status\x1b[0K\n\x1b[0K\n\x1b[0K\x1b[;H\n");
-      simpleRedBlackTreePrint(&objectStorage, stdout);
-      printf("\x1b[0J");
-      fflush(stdout);
+      elements = simpleRedBlackTreeGetElements(&objectStorage);
+
+      if( (elements != lastElements) || (elements > 0) ) {
+         printf("\x1b[;H");
+         printTimeStamp(stdout);
+         puts("Current Component Status\x1b[0K\n\x1b[0K\n\x1b[0K\x1b[;H\n");
+         simpleRedBlackTreePrint(&objectStorage, stdout);
+         printf("\x1b[0J");
+         fflush(stdout);
+      }
+
+      lastElements = elements;
    }
 
    ext_close(sd);
