@@ -1,40 +1,38 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: fractalgeneratorpackets.h 1731 2008-02-14 11:07:17Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: fractalgeneratorpackets.h 1336 2007-02-13 14:08:22Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef FRACTALGENERATORPACKETS_H
 #define FRACTALGENERATORPACKETS_H
 
+#include "tdtypes.h"
 #include "netdouble.h"
 
-#include <stdint.h>
-
-
-// Ensure consistent alignment!
-#pragma pack(push)
-#pragma pack(4)
-
 
 #define PPID_FGP 0x29097601
 
@@ -46,7 +44,7 @@ struct FGPCommonHeader
    uint8_t  Type;
    uint8_t  Flags;
    uint16_t Length;
-};
+} __attribute__((packed));
 
 
 
@@ -66,7 +64,7 @@ struct FGPParameter
    network_double_t C2Real;
    network_double_t C2Imag;
    network_double_t N;
-};
+} __attribute__((packed));
 
 #define FGD_MAX_POINTS 324
 
@@ -77,7 +75,7 @@ struct FGPData
    uint32_t        StartY;
    uint32_t        Points;
    uint32_t        Buffer[FGD_MAX_POINTS];
-};
+} __attribute__((packed));
 
 
 /**
@@ -102,16 +100,13 @@ struct FGPCookie
    FGPParameter Parameter;
    uint32_t     CurrentX;
    uint32_t     CurrentY;
-};
+} __attribute__((packed));
 
 
 struct FractalGeneratorStatus {
    struct FGPParameter Parameter;
    uint32_t            CurrentX;
    uint32_t            CurrentY;
-};
-
-
-#pragma pack(pop)
+} __attribute__((packed));
 
 #endif
