@@ -1,29 +1,32 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: hsdump.c 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: hsdump.c 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
-
 #include "tdtypes.h"
 #include "loglevel.h"
 #include "netutilities.h"
@@ -65,9 +68,10 @@ int main(int argc, char** argv)
    ssize_t                                    received;
    struct ST_CLASS(PeerListNode)              peerListNode;
    bool                                       moreData;
+   int                                        i;
 
    if(argc < 2) {
-      fprintf(stderr, "Usage: %s [Registrar]\n", argv[0]);
+      fprintf(stderr, "Usage: %s [Registrar] {-loglevel=Level} {-logfile=File} {-logappend=File} {-logcolor=on|off}\n", argv[0]);
       exit(1);
    }
 
@@ -75,6 +79,17 @@ int main(int argc, char** argv)
       fprintf(stderr, "ERROR: Bad registrar address <%s>\n", argv[1]);
       exit(1);
    }
+   for(i = 2;i < argc;i++) {
+      if(!(strncmp(argv[i], "-log" ,4))) {
+         if(initLogging(argv[i]) == false) {
+            exit(1);
+         }
+      }
+      else {
+         fprintf(stderr, "ERROR: Bad argument <%s>\n", argv[i]);
+      }
+   }
+   beginLogging();
 
    sd = ext_socket((checkIPv6() == true) ? AF_INET6 : AF_INET, SOCK_STREAM, IPPROTO_SCTP);
    if(sd < 0) {
@@ -133,7 +148,7 @@ int main(int argc, char** argv)
       message->ReceiverID                = 0;
       message->Checksum                  = ST_CLASS(poolHandlespaceManagementGetOwnershipChecksum)(&handlespace);
       if(rserpoolMessageSend(IPPROTO_SCTP,
-                              sd, 0, 0, 0, message) == false) {
+                              sd, 0, 0, 0, 0, message) == false) {
          fputs("ERROR: Sending Presence failed\n", stderr);
          rserpoolMessageDelete(message);
          ext_close(sd);
@@ -149,7 +164,7 @@ int main(int argc, char** argv)
       message->SenderID     = registrarID;
       message->ReceiverID   = UNDEFINED_REGISTRAR_IDENTIFIER;
       if(rserpoolMessageSend(IPPROTO_SCTP,
-                              sd, 0, 0, 0, message) == false) {
+                              sd, 0, 0, 0, 0, message) == false) {
          fputs("ERROR: Sending ListRequest failed\n", stderr);
          rserpoolMessageDelete(message);
          ext_close(sd);
@@ -165,7 +180,7 @@ int main(int argc, char** argv)
       message->SenderID     = registrarID;
       message->ReceiverID   = UNDEFINED_REGISTRAR_IDENTIFIER;
       if(rserpoolMessageSend(IPPROTO_SCTP,
-                              sd, 0, 0, 0, message) == false) {
+                              sd, 0, 0, 0, 0, message) == false) {
          fputs("ERROR: Sending HandleTableRequest failed\n", stderr);
          rserpoolMessageDelete(message);
          ext_close(sd);
@@ -185,65 +200,79 @@ int main(int argc, char** argv)
                                       received, sizeof(buffer), &message) == RSPERR_OKAY) {
                if(message->PPID == PPID_ENRP) {
                   if(message->Type == EHT_LIST_RESPONSE) {
-                     if( (!(message->Flags & EHF_LIST_RESPONSE_REJECT)) &&
-                         (message->PeerListPtr)) {
-                        peerListNodePtr = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(
-                                          &message->PeerListPtr->List);
-                        while(peerListNodePtr) {
-                           if(peerListNodePtr->Identifier != UNDEFINED_REGISTRAR_IDENTIFIER) {
-                              result = ST_CLASS(peerListManagementRegisterPeerListNode)(
-                                          &peerList,
-                                          peerListNodePtr->Identifier,
-                                          peerListNodePtr->Flags,
-                                          peerListNodePtr->AddressBlock,
-                                          getMicroTime(),
-                                          &newPeerListNode);
-                              if(result != RSPERR_OKAY) {
-                                 fputs("Failed to add peer ", stderr);
-                                 ST_CLASS(peerListNodePrint)(peerListNodePtr, stderr, ~0);
-                                 fputs(" to peer list\n", stderr);
+                     if(!(message->Flags & EHF_LIST_RESPONSE_REJECT)) {
+                        if(message->PeerListPtr) {
+                           peerListNodePtr = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(
+                                             &message->PeerListPtr->List);
+                           while(peerListNodePtr) {
+                              if(peerListNodePtr->Identifier != UNDEFINED_REGISTRAR_IDENTIFIER) {
+                                 result = ST_CLASS(peerListManagementRegisterPeerListNode)(
+                                             &peerList,
+                                             peerListNodePtr->Identifier,
+                                             peerListNodePtr->Flags,
+                                             peerListNodePtr->AddressBlock,
+                                             getMicroTime(),
+                                             &newPeerListNode);
+                                 if(result != RSPERR_OKAY) {
+                                    fputs("Failed to add peer ", stderr);
+                                    ST_CLASS(peerListNodePrint)(peerListNodePtr, stderr, ~0);
+                                    fputs(" to peer list\n", stderr);
+                                 }
                               }
+                              peerListNodePtr = ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(
+                                                   &message->PeerListPtr->List, peerListNodePtr);
                            }
-                           peerListNodePtr = ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(
-                                                &message->PeerListPtr->List, peerListNodePtr);
                         }
+                        else {
+                           puts("--- No peers ---");
+                        }
+                     }
+                     else {
+                        puts("--- ListRequest has been rejected ---");
                      }
                   }
                   else if(message->Type == EHT_HANDLE_TABLE_RESPONSE) {
-                     if( (!(message->Flags & EHF_HANDLE_TABLE_RESPONSE_REJECT)) &&
-                         (message->HandlespacePtr)) {
-                        poolElementNode = ST_CLASS(poolHandlespaceNodeGetFirstPoolElementOwnershipNode)(&message->HandlespacePtr->Handlespace);
-                        while(poolElementNode != NULL) {
-                           result = ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
-                                       &handlespace,
-                                       &poolElementNode->OwnerPoolNode->Handle,
-                                       poolElementNode->HomeRegistrarIdentifier,
-                                       poolElementNode->Identifier,
-                                       poolElementNode->RegistrationLife,
-                                       &poolElementNode->PolicySettings,
-                                       poolElementNode->UserTransport,
-                                       poolElementNode->RegistratorTransport,
-                                       -1, 0,
-                                       0,
-                                       &newPoolElementNode);
-                           if(result != RSPERR_OKAY) {
-                              fputs("Failed to register to pool ", stderr);
-                              poolHandlePrint(&message->Handle, stderr);
-                              fputs(" pool element ", stderr);
-                              ST_CLASS(poolElementNodePrint)(poolElementNode, stderr, PENPO_FULL);
-                              fputs(": ", stderr);
-                              rserpoolErrorPrint(result, stderr);
-                              fputs("\n", stderr);
+                     if(!(message->Flags & EHF_HANDLE_TABLE_RESPONSE_REJECT)) {
+                        if(message->HandlespacePtr) {
+                           poolElementNode = ST_CLASS(poolHandlespaceNodeGetFirstPoolElementOwnershipNode)(&message->HandlespacePtr->Handlespace);
+                           while(poolElementNode != NULL) {
+                              result = ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
+                                          &handlespace,
+                                          &poolElementNode->OwnerPoolNode->Handle,
+                                          poolElementNode->HomeRegistrarIdentifier,
+                                          poolElementNode->Identifier,
+                                          poolElementNode->RegistrationLife,
+                                          &poolElementNode->PolicySettings,
+                                          poolElementNode->UserTransport,
+                                          poolElementNode->RegistratorTransport,
+                                          -1, 0,
+                                          0,
+                                          &newPoolElementNode);
+                              if(result != RSPERR_OKAY) {
+                                 fputs("Failed to register to pool ", stderr);
+                                 poolHandlePrint(&message->Handle, stderr);
+                                 fputs(" pool element ", stderr);
+                                 ST_CLASS(poolElementNodePrint)(poolElementNode, stderr, PENPO_FULL);
+                                 fputs(": ", stderr);
+                                 rserpoolErrorPrint(result, stderr);
+                                 fputs("\n", stderr);
+                              }
+                              poolElementNode = ST_CLASS(poolHandlespaceNodeGetNextPoolElementOwnershipNode)(&message->HandlespacePtr->Handlespace, poolElementNode);
                            }
-                           poolElementNode = ST_CLASS(poolHandlespaceNodeGetNextPoolElementOwnershipNode)(&message->HandlespacePtr->Handlespace, poolElementNode);
-                        }
 
-                        moreData = (message->Flags & EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND);
-                        printf("Got %u pools, %u PEs => now having %u pools, %u PEs\n",
-                           (unsigned int)ST_CLASS(poolHandlespaceManagementGetPools)(message->HandlespacePtr),
-                           (unsigned int)ST_CLASS(poolHandlespaceManagementGetPoolElements)(message->HandlespacePtr),
-                           (unsigned int)ST_CLASS(poolHandlespaceManagementGetPools)(&handlespace),
-                           (unsigned int)ST_CLASS(poolHandlespaceManagementGetPoolElements)(&handlespace));
+                           moreData = (message->Flags & EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND);
+                           printf("Got %u pools, %u PEs => now having %u pools, %u PEs\n",
+                              (unsigned int)ST_CLASS(poolHandlespaceManagementGetPools)(message->HandlespacePtr),
+                              (unsigned int)ST_CLASS(poolHandlespaceManagementGetPoolElements)(message->HandlespacePtr),
+                              (unsigned int)ST_CLASS(poolHandlespaceManagementGetPools)(&handlespace),
+                              (unsigned int)ST_CLASS(poolHandlespaceManagementGetPoolElements)(&handlespace));
+                        }
+                        else {
+                           puts("--- No PEs ---");
+                        }
+                     }
+                     else {
+                        puts("--- HandleTableRequest has been rejected ---");
                      }
                      break;
                   }
@@ -282,5 +311,6 @@ int main(int argc, char** argv)
    ST_CLASS(peerListManagementDelete)(&peerList);
    ST_CLASS(poolHandlespaceManagementDelete)(&handlespace);
    ext_close(sd);
+   finishLogging();
    return(0);
 }
