@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: peerlistnode-template_impl.h 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: peerlistnode-template_impl.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 /* ###### Print ########################################################## */
@@ -179,17 +184,17 @@ void ST_CLASS(peerListNodeGetDescription)(
             peerListNode->Identifier,
             peerListNode->LastUpdateTimeStamp,
             (unsigned int)handlespaceChecksumFinish(peerListNode->OwnershipChecksum));
+   if(peerListNode->Flags & PENF_NEW) {
+      safestrcat(buffer, "[new]", bufferSize);
+   }
    if(!(peerListNode->Flags & PLNF_DYNAMIC)) {
-      safestrcat(buffer, "static", bufferSize);
+      safestrcat(buffer, "(static)", bufferSize);
    }
    else {
-      safestrcat(buffer, "dynamic", bufferSize);
+      safestrcat(buffer, "[dynamic]", bufferSize);
    }
    if(peerListNode->Flags & PLNF_FROM_PEER) {
-      safestrcat(buffer, "+fromPeer", bufferSize);
-   }
-   if(peerListNode->Flags & PLNF_MULTICAST) {
-      safestrcat(buffer, "+multicast", bufferSize);
+      safestrcat(buffer, "[fromPeer]", bufferSize);
    }
 
    if(peerListNode->Status & PLNS_LISTSYNC) {
