@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: poolelementnode-template_impl.h 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: poolelementnode-template_impl.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 /* ###### Initialize ##################################################### */
@@ -117,9 +122,15 @@ void ST_CLASS(poolElementNodeGetDescription)(
    char poolPolicySettingsDescription[512];
    char transportAddressDescription[1024];
 
-   snprintf(buffer, bufferSize, "$%08x", poolElementNode->Identifier);
+   snprintf(buffer, bufferSize, "$%08x flags=", poolElementNode->Identifier);
+   if(poolElementNode->Flags & PENF_NEW) {
+      safestrcat(buffer, "[new]", bufferSize);
+   }
+   if(poolElementNode->Flags & PENF_UPDATED) {
+      safestrcat(buffer, "[updated]", bufferSize);
+   }
    if(poolElementNode->Flags & PENF_MARKED) {
-      safestrcat(buffer, " [marked]", bufferSize);
+      safestrcat(buffer, "[marked]", bufferSize);
    }
    if(fields & (PENPO_CONNECTION|PENPO_CHECKSUM|PENPO_HOME_PR|PENPO_REGLIFE|PENPO_UR_REPORTS|PENPO_LASTUPDATE)) {
       safestrcat(buffer, "\n     ", bufferSize);
@@ -246,17 +257,22 @@ int ST_CLASS(poolElementNodeUpdate)(struct ST_CLASS(PoolElementNode)*       pool
                                     const struct ST_CLASS(PoolElementNode)* source)
 {
    poolElementNode->Flags &= ~PENF_MARKED;
-   if(poolPolicySettingsComparison(&poolElementNode->PolicySettings,
-                                   &source->PolicySettings) != 0) {
+   if( (poolPolicySettingsComparison(&poolElementNode->PolicySettings, &source->PolicySettings) != 0) ||
+       (poolElementNode->Degradation != 0) ) {
       /* ====== Update policy information ================================ */
       poolElementNode->PolicySettings = source->PolicySettings;
 
+      /* ====== Reset of degradation ===================================== */
+      poolElementNode->Degradation = 0;
+
       /* ====== Reduce virtual nodes counter, if necessary =============== */
       if(poolElementNode->VirtualCounter > poolElementNode->PolicySettings.Weight) {
          poolElementNode->VirtualCounter = poolElementNode->PolicySettings.Weight;
       }
+      poolElementNode->Flags |= PENF_UPDATED;
       return(1);
    }
+   poolElementNode->Flags &= ~PENF_UPDATED;
    return(0);
 }
 
