@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: poolhandlespacemanagement-template.h 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: poolhandlespacemanagement-template.h 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef INTERNAL_POOLTEMPLATE
@@ -64,6 +69,10 @@ void ST_CLASS(poolHandlespaceManagementNew)(
         void* disposerUserData);
 void ST_CLASS(poolHandlespaceManagementDelete)(
         struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
+void ST_CLASS(poolHandlespaceManagementGetDescription)(
+        const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+        char*                                             buffer,
+        const size_t                                      bufferSize);
 void ST_CLASS(poolHandlespaceManagementPrint)(
         const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
         FILE*                                             fd,
@@ -72,6 +81,7 @@ void ST_CLASS(poolHandlespaceManagementVerify)(
         struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
 void ST_CLASS(poolHandlespaceManagementClear)(
         struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
+
 HandlespaceChecksumType ST_CLASS(poolHandlespaceManagementGetHandlespaceChecksum)(
                            const struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
 HandlespaceChecksumType ST_CLASS(poolHandlespaceManagementGetOwnershipChecksum)(
@@ -89,6 +99,37 @@ size_t ST_CLASS(poolHandlespaceManagementGetPoolElementsOfConnection)(
           struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
           const int                                   socketDescriptor,
           const sctp_assoc_t                          assocID);
+
+struct ST_CLASS(PoolNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolNode)(
+                              struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
+struct ST_CLASS(PoolNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolNode)(
+                              struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                              struct ST_CLASS(PoolNode)*                  poolNode);
+
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolElementTimerNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementTimerNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode);
+
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceNodeGetFirstPoolElementConnectionNode)(
+                                     struct ST_CLASS(PoolHandlespaceNode)* poolHandlespaceNode);
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementConnectionNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode);
+
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolElementOwnershipNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementOwnershipNode)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode);
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetFirstPoolElementOwnershipNodeForIdentifier)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     const RegistrarIdentifierType               homeRegistrarIdentifier);
+struct ST_CLASS(PoolElementNode)* ST_CLASS(poolHandlespaceManagementGetNextPoolElementOwnershipNodeForSameIdentifier)(
+                                     struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+                                     struct ST_CLASS(PoolElementNode)*           poolElementNode);
+
 unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
                 struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
                 const struct PoolHandle*                    poolHandle,
@@ -108,6 +149,17 @@ unsigned int ST_CLASS(poolHandlespaceManagementRegisterPoolElementByPtr)(
                 const struct ST_CLASS(PoolElementNode)*     originalPoolElementNode,
                 const unsigned long long                    currentTimeStamp,
                 struct ST_CLASS(PoolElementNode)**          poolElementNode);
+
+void ST_CLASS(poolHandlespaceManagementUpdateOwnershipOfPoolElementNode)(
+              struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+              struct ST_CLASS(PoolElementNode)*           poolElementNode,
+              const RegistrarIdentifierType               newHomeRegistrarIdentifier);
+void ST_CLASS(poolHandlespaceManagementUpdateConnectionOfPoolElementNode)(
+        struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement,
+        struct ST_CLASS(PoolElementNode)*           poolElementNode,
+        const int                                   connectionSocketDescriptor,
+        const sctp_assoc_t                          connectionAssocID);
+
 unsigned long long ST_CLASS(poolHandlespaceManagementGetNextTimerTimeStamp)(
                       struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
 void ST_CLASS(poolHandlespaceManagementRestartPoolElementExpiryTimer)(
