@@ -1,41 +1,40 @@
-/*
- *  $Id: registrartable.c 1361 2007-05-09 13:55:39Z dreibh $
+/* $Id: registrartable.c 1744 2008-02-15 08:51:50Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * RSerPool implementation.
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and
+ * University of Essen, Institute of Computer Networking Technology.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * There are two mailinglists available at http://www.sctp.de/rserpool.html
- * which should be used for any discussion related to this implementation.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
- *
- * Purpose: Registrar Table
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
-
 #include "tdtypes.h"
 #include "registrartable.h"
 #include "rserpool-internals.h"
@@ -107,7 +106,7 @@ static void handleRegistrarAnnounceCallback(struct RegistrarTable* registrarTabl
                               &registrarTable->RegistrarList,
                               message->RegistrarIdentifier,
                               PLNF_DYNAMIC,
-                              message->TransportAddressBlockListPtr,
+                              message->PeerListNodePtr->AddressBlock,
                               getMicroTime(),
                               &peerListNode);
                   if(result == RSPERR_OKAY) {
@@ -243,9 +242,9 @@ struct RegistrarTable* registrarTableNew(struct Dispatcher*          dispatcher,
                return(false);
             }
 
-            if(!joinOrLeaveMulticastGroup(registrarTable->AnnounceSocket,
-                                          &registrarTable->AnnounceAddress,
-                                          true)) {
+            if(!multicastGroupControl(registrarTable->AnnounceSocket,
+                                      &registrarTable->AnnounceAddress,
+                                      true)) {
                LOG_WARNING
                fputs("Joining multicast group ",  stdlog);
                fputaddress(&registrarTable->AnnounceAddress.sa, true, stdlog);
@@ -291,12 +290,20 @@ struct RegistrarTable* registrarTableNew(struct Dispatcher*          dispatcher,
 /* ###### Destructor ##################################################### */
 void registrarTableDelete(struct RegistrarTable* registrarTable)
 {
+   struct SimpleRedBlackTreeNode* node;
+
    if(registrarTable != NULL) {
       if(registrarTable->AnnounceSocket >= 0) {
          fdCallbackDelete(&registrarTable->AnnounceSocketFDCallback);
          ext_close(registrarTable->AnnounceSocket);
          registrarTable->AnnounceSocket = -1;
       }
+      node = simpleRedBlackTreeGetFirst(&registrarTable->RegistrarAssocIDList);
+      while(node) {
+         CHECK(simpleRedBlackTreeRemove(&registrarTable->RegistrarAssocIDList, node) == node);
+         free(node);
+         node = simpleRedBlackTreeGetFirst(&registrarTable->RegistrarAssocIDList);
+      }
       simpleRedBlackTreeDelete(&registrarTable->RegistrarAssocIDList);
       ST_CLASS(peerListManagementDelete)(&registrarTable->RegistrarList);
       free(registrarTable);
@@ -329,7 +336,7 @@ static void addRegistrarAssocID(struct RegistrarTable* registrarTable,
 }
 
 
-/* ###### Remove registrar assoc ID to list ############################## */
+/* ###### Remove registrar assoc ID from list ############################ */
 static void removeRegistrarAssocID(struct RegistrarTable* registrarTable,
                                    int                    registrarHuntFD,
                                    sctp_assoc_t           assocID)
@@ -401,7 +408,7 @@ static int selectRegistrar(struct RegistrarTable*   registrarTable,
          fputs("\n", stdlog);
          LOG_END
 
-         /* ====== Get registrar's entry ======================== */
+         /* ====== Get registrar's entry ================================= */
          struct TransportAddressBlock* registrarAddressBlock = (struct TransportAddressBlock*)malloc(transportAddressBlockGetSize(n));
          if(registrarAddressBlock) {
             transportAddressBlockNew(registrarAddressBlock,
@@ -419,12 +426,20 @@ static int selectRegistrar(struct RegistrarTable*   registrarTable,
             free(registrarAddressBlock);
          }
          free(peerAddressArray);
-      }
 
-      sd = registrarTablePeelOffRegistrarAssocID(registrarTable,
-                                                 registrarHuntFD,
-                                                 registrarHuntMessageBuffer,
-                                                 assocID);
+         /* ====== Peel-off association ================================== */
+         sd = registrarTablePeelOffRegistrarAssocID(registrarTable,
+                                                    registrarHuntFD,
+                                                    registrarHuntMessageBuffer,
+                                                    assocID);
+      }
+      else {
+         LOG_VERBOSE2
+         fprintf(stdlog, "Cannot find our peer addresses of assoc %u (already broken) -> removing it\n",
+                 (unsigned int)assocID);
+         LOG_END
+         removeRegistrarAssocID(registrarTable, registrarHuntFD, assocID);
+      }
    }
    return(sd);
 }
@@ -441,8 +456,15 @@ int registrarTablePeelOffRegistrarAssocID(struct RegistrarTable* registrarTable,
       LOG_VERBOSE2
       fprintf(stdlog, "Assoc %u peeled off from registrar hunt socket\n", (unsigned int)assocID);
       LOG_END
-      removeRegistrarAssocID(registrarTable, registrarHuntFD, assocID);
    }
+   else {
+      LOG_ERROR
+      fprintf(stdlog, "Assoc %u peel-off failed: %s\n", assocID, strerror(errno));
+      LOG_END_FATAL
+      sendabort(registrarHuntFD, assocID);
+   }
+   /* In any case, the registrar association is broken -> remove it. */
+   removeRegistrarAssocID(registrarTable, registrarHuntFD, assocID);
    return(sd);
 }
 
@@ -657,7 +679,7 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
 
    registrarTable->OutstandingConnects = 0;
 
-   peerListNode = ST_CLASS(peerListGetRandomPeerNode)(&registrarTable->RegistrarList.List);
+   peerListNode = ST_CLASS(peerListManagementGetRandomPeerListNode)(&registrarTable->RegistrarList);
    if(peerListNode) {
       lastRegistrarIdentifier   = peerListNode->Identifier;
       lastTransportAddressBlock = transportAddressBlockDuplicate(peerListNode->AddressBlock);
@@ -702,9 +724,9 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
             LOG_END
 
             if( (registrarTable->AnnounceAddress.sa.sa_family != 0) &&
-                (!joinOrLeaveMulticastGroup(registrarTable->AnnounceSocket,
-                                            &registrarTable->AnnounceAddress,
-                                            true)) ) {
+                (!multicastGroupControl(registrarTable->AnnounceSocket,
+                                        &registrarTable->AnnounceAddress,
+                                        true)) ) {
                LOG_WARNING
                fputs("Joining multicast group ",  stdlog);
                fputaddress(&registrarTable->AnnounceAddress.sa, true, stdlog);
@@ -802,6 +824,8 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
                   }
 
                   /* ====== Is there a connection to a registrar? ======== */
+                  /* This function will randomly chose a registrar from the
+                     registrar list. */
                   sd = selectRegistrar(registrarTable, registrarHuntFD, registrarHuntMessageBuffer,
                                        registrarIdentifier);
                   if(sd >= 0) {
@@ -826,15 +850,6 @@ int registrarTableGetRegistrar(struct RegistrarTable*   registrarTable,
                      LOG_END
                      return(sd);
                   }
-
-                  LOG_WARNING
-                  fprintf(stdlog, "Got crap via assoc %u -> aborting it\n", (unsigned int)assocID);
-                  LOG_END
-                  flags = 0;
-                  messageBufferRead(registrarHuntMessageBuffer, registrarHuntFD, &flags,
-                                    NULL, 0,
-                                    NULL, &assocID, NULL, 0);
-                  sendabort(registrarHuntFD, assocID);
                }
             }
          }
