@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: rspbasicmode.c 1741 2008-02-14 17:12:54Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: rspbasicmode.c 1436 2007-06-06 12:30:19Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "tdtypes.h"
@@ -39,6 +43,8 @@
 #include "componentstatusreporter.h"
 #endif
 
+#include <stdio.h>
+
 
 struct ASAPInstance*       gAsapInstance = NULL;
 struct Dispatcher          gDispatcher;
@@ -186,20 +192,11 @@ void rsp_cleanup()
    int i;
 
    if(gAsapInstance) {
-#ifdef ENABLE_CSP
-      /* ====== Remove Component Status Reporter ========================= */
-      if(gCSPReporter) {
-         cspReporterDelete(gCSPReporter);
-         free(gCSPReporter);
-         gCSPReporter = NULL;
-      }
-#endif
-
       /* ====== Clean-up RSerPool Socket descriptor storage ============== */
       CHECK(rsp_unmapsocket(STDOUT_FILENO) == 0);
       CHECK(rsp_unmapsocket(STDIN_FILENO) == 0);
       CHECK(rsp_unmapsocket(STDERR_FILENO) == 0);
-      for(i = 1;i < FD_SETSIZE;i++) {
+      for(i = 1;i < (int)FD_SETSIZE;i++) {
          if(identifierBitmapAllocateSpecificID(gRSerPoolSocketAllocationBitmap, i) < 0) {
             LOG_WARNING
             fprintf(stdlog, "RSerPool socket %d is still registered -> closing it\n", i);
@@ -207,16 +204,30 @@ void rsp_cleanup()
             rsp_close(i);
          }
       }
+
+      /* ====== Clean-up ASAP Instance, CSP Reported and Dispatcher ====== */
+      asapInstanceDelete(gAsapInstance);
+      gAsapInstance = NULL;
+
+      /* ====== Remove Component Status Reporter ========================= */
+#ifdef ENABLE_CSP
+      if(gCSPReporter) {
+         cspReporterDelete(gCSPReporter);
+         free(gCSPReporter);
+         gCSPReporter = NULL;
+      }
+#endif
+
+      /* ====== Remove Dispatcher ======================================== */
+      dispatcherDelete(&gDispatcher);
+      threadSafetyDelete(&gRSerPoolSocketSetMutex);
+      threadSafetyDelete(&gThreadSafety);
+
+      /* ====== Remove socket set ======================================== */
       simpleRedBlackTreeDelete(&gRSerPoolSocketSet);
       identifierBitmapDelete(gRSerPoolSocketAllocationBitmap);
       gRSerPoolSocketAllocationBitmap = NULL;
 
-      /* ====== Clean-up ASAP instance and Dispatcher ==================== */
-      asapInstanceDelete(gAsapInstance);
-      gAsapInstance = NULL;
-      dispatcherDelete(&gDispatcher);
-      threadSafetyDelete(&gRSerPoolSocketSetMutex);
-      threadSafetyDelete(&gThreadSafety);
    }
    finishLogging();
 }
@@ -306,9 +317,9 @@ static int poolElementNodeToAddrInfo(const struct ST_CLASS(PoolElementNode)* poo
                default:
                   LOG_ERROR
                   fprintf(stdlog, "Bad address type #%d\n",
-                           poolElementNode->UserTransport->AddressArray[i].sa.sa_family);
+                          poolElementNode->UserTransport->AddressArray[i].sa.sa_family);
                   LOG_END_FATAL
-                  result = EAI_ADDRFAMILY;
+                  result = EAI_FAMILY;
                 break;
             }
          }
@@ -323,47 +334,51 @@ static int poolElementNodeToAddrInfo(const struct ST_CLASS(PoolElementNode)* poo
 }
 
 
+// ###### Conversion function from PoolElementNode to rsp_addrinfo ####### */
+static unsigned int convertPoolElementNode(
+                       const struct ST_CLASS(PoolElementNode)* poolElementNode,
+                       void*                                   ptr)
+{
+   return(poolElementNodeToAddrInfo(poolElementNode, (struct rsp_addrinfo**)ptr));
+}
+
+
 /* ###### Handle resolution ############################################## */
 int rsp_getaddrinfo_tags(const unsigned char*  poolHandle,
                          const size_t          poolHandleSize,
                          struct rsp_addrinfo** rspAddrInfo,
                          const size_t          items,
+                         const unsigned int    staleCacheValue,
                          struct TagItem*       tags)
 {
-   struct PoolHandle                 myPoolHandle;
-   struct ST_CLASS(PoolElementNode)* poolElementNodeArray[MAX_MAX_HANDLE_RESOLUTION_ITEMS];
-   size_t                            poolElementNodes;
-   struct rsp_addrinfo**             lastAddrInfo;
-   unsigned int                      hresResult;
-   int                               result;
-   size_t                            n;
+   struct PoolHandle myPoolHandle;
+   void*             addrInfoArray[MAX_MAX_HANDLE_RESOLUTION_ITEMS];
+   size_t            addrInfos;
+   unsigned int      hresResult;
+   int               result;
+   size_t            n;
 
    *rspAddrInfo = NULL;
    if(gAsapInstance) {
       poolHandleNew(&myPoolHandle, poolHandle, poolHandleSize);
+      addrInfos = max(1, min((size_t)items, MAX_MAX_HANDLE_RESOLUTION_ITEMS));
 
-      poolElementNodes = max(1, min((size_t)items, MAX_MAX_HANDLE_RESOLUTION_ITEMS));
       hresResult = asapInstanceHandleResolution(
                       gAsapInstance,
                       &myPoolHandle,
-                      (struct ST_CLASS(PoolElementNode)**)&poolElementNodeArray,
-                      &poolElementNodes);
+                      (void**)&addrInfoArray,
+                      &addrInfos,
+                      convertPoolElementNode,
+                      1000ULL * staleCacheValue);
       if(hresResult == RSPERR_OKAY) {
-         lastAddrInfo = rspAddrInfo;
-         for(n = 0;n < poolElementNodes;n++) {
-            result = poolElementNodeToAddrInfo(poolElementNodeArray[n], lastAddrInfo);
-            if(result != 0) {
-               break;
-            }
-            if(*rspAddrInfo != NULL) {
-               lastAddrInfo = &(*lastAddrInfo)->ai_next;
+         if(addrInfos > 0) {
+            for(n = 0;n < addrInfos - 1;n++) {
+               ((struct rsp_addrinfo*)addrInfoArray[n])->ai_next =
+                  (struct rsp_addrinfo*)addrInfoArray[n + 1];
             }
+            *rspAddrInfo = (struct rsp_addrinfo*)addrInfoArray[0];
          }
-         if(result == 0) {
-            return(n);
-         }
-         rsp_freeaddrinfo(*rspAddrInfo);
-         *rspAddrInfo = NULL;
+         return(addrInfos);
       }
       else {
          if(hresResult == RSPERR_NOT_FOUND) {
@@ -389,9 +404,10 @@ int rsp_getaddrinfo_tags(const unsigned char*  poolHandle,
 int rsp_getaddrinfo(const unsigned char*  poolHandle,
                     const size_t          poolHandleSize,
                     struct rsp_addrinfo** rspAddrInfo,
-                    const size_t          items)
+                    const size_t          items,
+                    const unsigned int    staleCacheValue)
 {
-   return(rsp_getaddrinfo_tags(poolHandle, poolHandleSize, rspAddrInfo, items, NULL));
+   return(rsp_getaddrinfo_tags(poolHandle, poolHandleSize, rspAddrInfo, items, staleCacheValue, NULL));
 }
 
 
@@ -458,7 +474,7 @@ unsigned int rsp_pe_registration_tags(const unsigned char* poolHandle,
             &myPoolElementNode,
             rspAddrInfo->ai_pe_id,
             0,
-            1000ULL * registrationLife,
+            registrationLife,
             &myPolicySettings,
             myTransportAddressBlock,
             NULL,
