@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: rspenhancedmode.c 1741 2008-02-14 17:12:54Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: rspenhancedmode.c 1435 2007-06-05 13:20:58Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "tdtypes.h"
@@ -259,7 +263,7 @@ int rsp_mapsocket(int sd, int toSD)
    struct RSerPoolSocket* rserpoolSocket;
 
    /* ====== Check for problems ========================================== */
-   if((sd < 0) || (sd >= FD_SETSIZE)) {
+   if((sd < 0) || (sd >= (int)FD_SETSIZE)) {
       errno = EINVAL;
       return(-1);
    }
@@ -412,7 +416,7 @@ int rsp_select(int n, fd_set* readfds, fd_set* writefds, fd_set* exceptfds,
    int           i;
 
    /* ====== Check for problems ========================================== */
-   if(n > FD_SETSIZE) {
+   if(n > (int)FD_SETSIZE) {
       errno = EINVAL;
       return(-1);
    }
@@ -662,6 +666,7 @@ int rsp_deregister(int sd, int flags)
 int rsp_connect_tags(int                  sd,
                      const unsigned char* poolHandle,
                      const size_t         poolHandleSize,
+                     const unsigned int   staleCacheValue,
                      struct TagItem*      tags)
 {
    struct RSerPoolSocket* rserpoolSocket;
@@ -680,22 +685,16 @@ int rsp_connect_tags(int                  sd,
          rserpoolSocket->ConnectedSession = session;
          /* Session Change events are only useful for handling
             multiple incoming sessions. */
-         rserpoolSocket->Notifications.EventMask &= ~ NET_SESSION_CHANGE;
+         rserpoolSocket->Notifications.EventMask &= ~NET_SESSION_CHANGE;
 
          /* ====== Connect to a PE ======================================= */
          /* Do not signalize successful failover, since this is the first
             connection establishment */
          oldEventMask = rserpoolSocket->Notifications.EventMask;
          rserpoolSocket->Notifications.EventMask = 0;
-         if(rsp_forcefailover_tags(rserpoolSocket->Descriptor, tags) == 0) {
-            result = 0;
-         }
-         else {
-            deleteSession(rserpoolSocket, session);
-            errno = EIO;
-         }
+         rsp_forcefailover_tags(rserpoolSocket->Descriptor, FFF_NONE, staleCacheValue, tags);
          rserpoolSocket->Notifications.EventMask = oldEventMask;
-
+         result = 0;
       }
       else {
          errno = ENOMEM;
@@ -717,9 +716,10 @@ int rsp_connect_tags(int                  sd,
 /* ###### RSerPool socket connect() implementation ####################### */
 int rsp_connect(int                  sd,
                 const unsigned char* poolHandle,
-                const size_t         poolHandleSize)
+                const size_t         poolHandleSize,
+                const unsigned int   staleCacheValue)
 {
-   return(rsp_connect_tags(sd, poolHandle, poolHandleSize, NULL));
+   return(rsp_connect_tags(sd, poolHandle, poolHandleSize, staleCacheValue, NULL));
 }
 
 
@@ -805,7 +805,11 @@ static int connectToPE(struct RSerPoolSocket* rserpoolSocket,
                  rserpoolSocket->ConnectedSession->SessionID);
          LOG_END
 
+#ifdef HAVE_CONNECTX_WITH_ID
+         result = sctp_connectx(sd, (struct sockaddr*)destinationAddressArray, destinationAddresses, NULL);
+#else
          result = sctp_connectx(sd, (struct sockaddr*)destinationAddressArray, destinationAddresses);
+#endif
          if((result == 0) || (errno == EINPROGRESS)) {
             ufds.fd     = sd;
             ufds.events = POLLIN;
@@ -823,9 +827,9 @@ static int connectToPE(struct RSerPoolSocket* rserpoolSocket,
          fprintf(stdlog, "Failed to establish connection to pool element $%08x (", peIdentifier);
          poolHandlePrint(&rserpoolSocket->ConnectedSession->Handle, stdlog);
          fprintf(stdlog, "/$%08x on RSerPool socket %u, socket %d, session %u)\n",
-               peIdentifier,
-               rserpoolSocket->Descriptor, rserpoolSocket->Socket,
-               rserpoolSocket->ConnectedSession->SessionID);
+                 peIdentifier,
+                 rserpoolSocket->Descriptor, rserpoolSocket->Socket,
+                 rserpoolSocket->ConnectedSession->SessionID);
          LOG_END
       }
       ext_close(sd);
@@ -849,9 +853,31 @@ int rsp_has_cookie(int sd)
 }
 
 
+/* ###### Report PE as failed ############################################ */
+static void rsp_send_failure_report(struct RSerPoolSocket* rserpoolSocket,
+                                    struct TagItem*        tags)
+{
+   LOG_ACTION
+   fputs("Reporting failure of ", stdlog);
+   poolHandlePrint(&rserpoolSocket->ConnectedSession->Handle, stdlog);
+   fprintf(stdlog, "/$%08x on RSerPool socket %u, socket %d, session %u\n",
+           rserpoolSocket->ConnectedSession->ConnectedPE,
+           rserpoolSocket->Descriptor, rserpoolSocket->Socket,
+           rserpoolSocket->ConnectedSession->SessionID);
+   LOG_END
+   rsp_pe_failure_tags((unsigned char*)&rserpoolSocket->ConnectedSession->Handle.Handle,
+                       rserpoolSocket->ConnectedSession->Handle.Size,
+                       rserpoolSocket->ConnectedSession->ConnectedPE,
+                       tags);
+   rserpoolSocket->ConnectedSession->ConnectedPE = 0;
+}
+
+
 /* ###### Make failover ################################################## */
-int rsp_forcefailover_tags(int             sd,
-                           struct TagItem* tags)
+int rsp_forcefailover_tags(int                sd,
+                           const unsigned int flags,
+                           const unsigned int staleCacheValue,
+                           struct TagItem*    tags)
 {
    struct RSerPoolSocket*   rserpoolSocket;
    struct rsp_addrinfo*     rspAddrInfo;
@@ -885,21 +911,10 @@ int rsp_forcefailover_tags(int             sd,
    notificationQueueClear(&rserpoolSocket->Notifications);
 
    /* ====== Report failure ============================================== */
-   if((rserpoolSocket->ConnectedSession->ConnectedPE != 0) &&
+   if((flags & FFF_UNREACHABLE) &&
+      (rserpoolSocket->ConnectedSession->ConnectedPE != 0) &&
       (rserpoolSocket->ConnectedSession->Handle.Size > 0)) {
-      LOG_ACTION
-      fputs("Reporting failure of ", stdlog);
-      poolHandlePrint(&rserpoolSocket->ConnectedSession->Handle, stdlog);
-      fprintf(stdlog, "/$%08x on RSerPool socket %u, socket %d, session %u\n",
-              rserpoolSocket->ConnectedSession->ConnectedPE,
-              rserpoolSocket->Descriptor, rserpoolSocket->Socket,
-              rserpoolSocket->ConnectedSession->SessionID);
-      LOG_END
-      rsp_pe_failure_tags((unsigned char*)&rserpoolSocket->ConnectedSession->Handle.Handle,
-                          rserpoolSocket->ConnectedSession->Handle.Size,
-                          rserpoolSocket->ConnectedSession->ConnectedPE,
-                          tags);
-      rserpoolSocket->ConnectedSession->ConnectedPE = 0;
+      rsp_send_failure_report(rserpoolSocket, tags);
    }
 
    /* ====== Abort association =========================================== */
@@ -927,22 +942,22 @@ int rsp_forcefailover_tags(int             sd,
       LOG_END
       result = rsp_getaddrinfo_tags((unsigned char*)&rserpoolSocket->ConnectedSession->Handle.Handle,
                                     rserpoolSocket->ConnectedSession->Handle.Size,
-                                    &rspAddrInfo, 1,
+                                    &rspAddrInfo, 1, staleCacheValue,
                                     tags);
       if(result > 0) {
          if(rspAddrInfo->ai_protocol == rserpoolSocket->SocketProtocol) {
             /* ====== Establish connection ================================== */
             rserpoolSocket->Socket = connectToPE(rserpoolSocket,
-                                                rspAddrInfo->ai_pe_id,
-                                                rspAddrInfo->ai_addr,
-                                                rspAddrInfo->ai_addrs);
+                                                 rspAddrInfo->ai_pe_id,
+                                                 rspAddrInfo->ai_addr,
+                                                 rspAddrInfo->ai_addrs);
             if(rserpoolSocket->Socket >= 0) {
                success = true;
                rserpoolSocket->ConnectedSession->ConnectionTimeStamp = getMicroTime();
                rserpoolSocket->ConnectedSession->ConnectedPE         = rspAddrInfo->ai_pe_id;
                sessionStorageUpdateSession(&rserpoolSocket->SessionSet,
-                                             rserpoolSocket->ConnectedSession,
-                                             0);
+                                           rserpoolSocket->ConnectedSession,
+                                           0);
             }
 
             /* ====== Free handle resolution result ====================== */
@@ -979,6 +994,7 @@ int rsp_forcefailover_tags(int             sd,
                LOG_ACTION
                fputs("Connection establishment was not possible\n", stdlog);
                LOG_END
+               rsp_send_failure_report(rserpoolSocket, tags);
             }
          }
          else {
@@ -1020,9 +1036,11 @@ int rsp_forcefailover_tags(int             sd,
 
 
 /* ###### Make failover ################################################## */
-int rsp_forcefailover(int sd)
+int rsp_forcefailover(int                sd,
+                      const unsigned int flags,
+                      const unsigned int staleCacheValue)
 {
-   return(rsp_forcefailover_tags(sd, NULL));
+   return(rsp_forcefailover_tags(sd, flags, staleCacheValue, NULL));
 }
 
 
@@ -1035,6 +1053,7 @@ ssize_t rsp_sendmsg(int                sd,
                     uint32_t           sctpPPID,
                     uint16_t           sctpStreamID,
                     uint32_t           sctpTimeToLive,
+                    uint16_t           sctpFlags,
                     int                timeout)
 {
    struct RSerPoolSocket*   rserpoolSocket;
@@ -1060,12 +1079,12 @@ ssize_t rsp_sendmsg(int                sd,
                              msg_flags,
 #endif
                              NULL, 0,
-                             ntohl(sctpPPID), session->AssocID, sctpStreamID, sctpTimeToLive,
+                             ntohl(sctpPPID), session->AssocID, sctpStreamID, sctpTimeToLive, sctpFlags,
                              (timeout >= 0) ? (1000ULL * timeout) : 0);
          if((result < 0) && (errno != EAGAIN)) {
             LOG_ACTION
-            fprintf(stdlog, "Session failure during send on RSerPool socket %d, session %u. Failover necessary\n",
-                    rserpoolSocket->Descriptor, session->SessionID);
+            fprintf(stdlog, "Session failure during send on RSerPool socket %d, session %u: %s. Failover necessary\n",
+                    rserpoolSocket->Descriptor, session->SessionID, strerror(errno));
             LOG_END
 
             /* ====== Terminate association and notify application ======= */
@@ -1379,7 +1398,7 @@ ssize_t rsp_send(int sd, const void* buffer, size_t bufferLength, int flags)
    flags |= MSG_NOSIGNAL;
 #endif
    if(rserpoolSocket->SessionAllocationBitmap != NULL) {
-      return(rsp_sendmsg(sd, buffer, bufferLength, flags, 0, 0, 0, 0, -1));
+      return(rsp_sendmsg(sd, buffer, bufferLength, flags, 0, 0, 0, 0, 0, -1));
    }
    return(ext_send(rserpoolSocket->Socket, buffer, bufferLength, flags));
 }
@@ -1391,7 +1410,7 @@ ssize_t rsp_write(int fd, const char* buffer, size_t bufferLength)
    struct RSerPoolSocket* rserpoolSocket;
    GET_RSERPOOL_SOCKET(rserpoolSocket, fd);
    if(rserpoolSocket->SessionAllocationBitmap != NULL) {
-      return(rsp_sendmsg(fd, buffer, bufferLength, 0, 0, 0, 0, 0, -1));
+      return(rsp_sendmsg(fd, buffer, bufferLength, 0, 0, 0, 0, 0, 0, -1));
    }
    return(ext_write(rserpoolSocket->Socket, buffer, bufferLength));
 }
@@ -1429,7 +1448,7 @@ ssize_t rsp_send_cookie(int                  sd,
          result = rserpoolMessageSend(IPPROTO_SCTP,
                                       rserpoolSocket->Socket,
                                       session->AssocID,
-                                      0,
+                                      0, 0,
                                       (1000ULL * timeout),
                                       message);
          threadSafetyUnlock(&rserpoolSocket->Mutex);
