@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: server.cc 1726 2008-02-14 09:56:55Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: server.cc 1436 2007-06-06 12:30:19Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "rserpool.h"
@@ -30,18 +34,22 @@
 #include "standardservices.h"
 #include "fractalgeneratorservice.h"
 #include "calcappservice.h"
+#include "scriptingservice.h"
 #ifdef ENABLE_CSP
 #include "componentstatuspackets.h"
 #endif
 
+#include <math.h>
 
-#define SERVICE_ECHO     1
-#define SERVICE_DISCARD  2
-#define SERVICE_DAYTIME  3
-#define SERVICE_CHARGEN  4
-#define SERVICE_PINGPONG 5
-#define SERVICE_FRACTAL  6
-#define SERVICE_CALCAPP  7
+
+#define SERVICE_ECHO      1
+#define SERVICE_DISCARD   2
+#define SERVICE_DAYTIME   3
+#define SERVICE_CHARGEN   4
+#define SERVICE_PINGPONG  5
+#define SERVICE_FRACTAL   6
+#define SERVICE_CALCAPP   7
+#define SERVICE_SCRIPTING 8
 
 
 /* ###### Main program ################################################### */
@@ -56,6 +64,8 @@ int main(int argc, char** argv)
    unsigned int        runtimeLimit  = 0;
    unsigned int        service       = SERVICE_ECHO;
    const char*         poolHandle    = NULL;
+   bool                policyChanged = false;
+   bool                quiet         = false;
 
    /* ====== Read parameters ============================================= */
    rsp_initinfo(&info);
@@ -180,6 +190,7 @@ int main(int argc, char** argv)
             fprintf(stderr, "ERROR: Bad policy setting <%s>!\n", argv[i]);
             exit(1);
          }
+         policyChanged = true;
       }
       else if(!(strncmp(argv[i], "-runtime=" ,9))) {
          runtimeLimit = atol((const char*)&argv[i][9]);
@@ -205,24 +216,32 @@ int main(int argc, char** argv)
       else if(!(strcmp(argv[i], "-calcapp"))) {
          service = SERVICE_CALCAPP;
       }
+      else if(!(strcmp(argv[i], "-scripting"))) {
+         service = SERVICE_SCRIPTING;
+      }
+      else if(!(strcmp(argv[i], "-quiet"))) {
+         quiet = true;
+      }
    }
 
 
    /* ====== Print startup message ======================================= */
-   printf("Starting service ");
-   if(service == SERVICE_ECHO) {
-      printf("Echo");
+   if(!quiet) {
+      printf("Starting service ");
+      if(service == SERVICE_ECHO) {
+         printf("Echo");
+      }
+      else if(service == SERVICE_DISCARD) {
+         printf("Discard");
+      }
+      else if(service == SERVICE_DAYTIME) {
+         printf("Daytime");
+      }
+      else if(service == SERVICE_CHARGEN) {
+         printf("Character Generator");
+      }
+      puts("...");
    }
-   else if(service == SERVICE_DISCARD) {
-      printf("Discard");
-   }
-   else if(service == SERVICE_DAYTIME) {
-      printf("Daytime");
-   }
-   else if(service == SERVICE_CHARGEN) {
-      printf("Character Generator");
-   }
-   puts("...");
 
 
    /* ====== Start requested service ===================================== */
@@ -231,7 +250,7 @@ int main(int argc, char** argv)
       echoServer.poolElement("Echo Server - Version 1.0",
                              (poolHandle != NULL) ? poolHandle : "EchoPool",
                              &info, &loadInfo,
-                             reregInterval, runtimeLimit,
+                             reregInterval, runtimeLimit, quiet,
                              (struct TagItem*)&tags);
    }
    else if(service == SERVICE_DISCARD) {
@@ -239,7 +258,7 @@ int main(int argc, char** argv)
       discardServer.poolElement("Discard Server - Version 1.0",
                                 (poolHandle != NULL) ? poolHandle : "DiscardPool",
                                 &info, &loadInfo,
-                                reregInterval, runtimeLimit,
+                                reregInterval, runtimeLimit, quiet,
                                 (struct TagItem*)&tags);
    }
    else if(service == SERVICE_DAYTIME) {
@@ -247,7 +266,7 @@ int main(int argc, char** argv)
       daytimeServer.poolElement("Daytime Server - Version 1.0",
                                 (poolHandle != NULL) ? poolHandle : "DaytimePool",
                                 &info, &loadInfo,
-                                reregInterval, runtimeLimit,
+                                reregInterval, runtimeLimit, quiet,
                                 (struct TagItem*)&tags);
    }
    else if(service == SERVICE_CHARGEN) {
@@ -264,7 +283,7 @@ int main(int argc, char** argv)
                                   maxThreads,
                                   CharGenServer::charGenServerFactory,
                                   NULL, NULL, NULL, NULL, NULL,
-                                  reregInterval, runtimeLimit,
+                                  reregInterval, runtimeLimit, quiet,
                                   (struct TagItem*)&tags);
    }
    else if(service == SERVICE_PINGPONG) {
@@ -287,18 +306,43 @@ int main(int argc, char** argv)
                                  PingPongServer::pingPongServerFactory,
                                  NULL, NULL, NULL, NULL,
                                  (void*)&settings,
-                                 reregInterval, runtimeLimit,
+                                 reregInterval, runtimeLimit, quiet,
                                  (struct TagItem*)&tags);
    }
    else if(service == SERVICE_FRACTAL) {
       size_t maxThreads = 4;
       FractalGeneratorServer::FractalGeneratorServerSettings settings;
-      settings.TestMode     = false;
-      settings.FailureAfter = 0;
+      settings.TestMode         = false;
+      settings.FailureAfter     = 0;
+      settings.TransmitTimeout  = 2500;
+      settings.DataMaxTime      = 1000000;
+      settings.CookieMaxTime    = 1000000;
+      settings.CookieMaxPackets = 10;
       for(int i = 1;i < argc;i++) {
          if(!(strcmp(argv[i], "-fgptestmode"))) {
             settings.TestMode = true;
          }
+         else if(!(strncmp(argv[i], "-fgptransmittimeout=", 20))) {
+            settings.TransmitTimeout = atol((const char*)&argv[i][20]);
+         }
+         else if(!(strncmp(argv[i], "-fgpdatamaxtime=", 16))) {
+            settings.DataMaxTime = 1000ULL * atol((const char*)&argv[i][16]);
+            if(settings.DataMaxTime < 100000) {
+               settings.DataMaxTime = 100000;
+            }
+         }
+         else if(!(strncmp(argv[i], "-fgpcookiemaxtime=", 18))) {
+            settings.CookieMaxTime = 1000ULL * atol((const char*)&argv[i][18]);
+            if(settings.CookieMaxTime < 100000) {
+               settings.CookieMaxTime = 100000;
+            }
+         }
+         else if(!(strncmp(argv[i], "-fgpcookiemaxpackets=", 21))) {
+            settings.CookieMaxPackets = atol((const char*)&argv[i][21]);
+            if(settings.CookieMaxPackets < 1) {
+               settings.CookieMaxPackets = 1;
+            }
+         }
          else if(!(strncmp(argv[i], "-fgpmaxthreads=", 15))) {
             maxThreads = atol((const char*)&argv[i][15]);
          }
@@ -315,7 +359,7 @@ int main(int argc, char** argv)
                                  FractalGeneratorServer::fractalGeneratorPrintParameters,
                                  NULL, NULL, NULL,
                                  (void*)&settings,
-                                 reregInterval, runtimeLimit,
+                                 reregInterval, runtimeLimit, quiet,
                                  (struct TagItem*)&tags);
    }
    else if(service == SERVICE_CALCAPP) {
@@ -383,9 +427,44 @@ int main(int argc, char** argv)
       calcAppServer.poolElement("CalcApp Server - Version 1.0",
                                 (poolHandle != NULL) ? poolHandle : "CalcAppPool",
                                 &info, &loadInfo,
-                                reregInterval, runtimeLimit,
+                                reregInterval, runtimeLimit, quiet,
                                 (struct TagItem*)&tags);
    }
+   else if(service == SERVICE_SCRIPTING) {
+      size_t maxThreads = 1;
+      ScriptingServer::ScriptingServerSettings settings;
+      settings.KeepTempDirs    = false;
+      settings.TransmitTimeout = 60000;
+      for(int i = 1;i < argc;i++) {
+         if(!(strncmp(argv[i], "-ssmaxthreads=", 14))) {
+            maxThreads = atol((const char*)&argv[i][14]);
+            if(maxThreads < 1) {
+               maxThreads = 1;
+            }
+         }
+         else if(!(strcmp(argv[i], "-sskeeptempdirs"))) {
+            settings.KeepTempDirs = true;
+         }
+         else if(!(strncmp(argv[i], "-sstransmittimeout=", 19))) {
+            settings.TransmitTimeout = atol((const char*)&argv[i][19]);
+         }
+      }
+      if(!policyChanged) {
+         loadInfo.rli_load_degradation = (unsigned int)rint((1.0 / maxThreads) * (double)PPV_MAX_LOAD_DEGRADATION);
+         loadInfo.rli_policy = PPT_LEASTUSED_DEGRADATION;
+      }
+
+      TCPLikeServer::poolElement("Scripting Server - Version 1.0",
+                                 (poolHandle != NULL) ? poolHandle : "ScriptingPool",
+                                 &info, &loadInfo,
+                                 maxThreads,
+                                 ScriptingServer::scriptingServerFactory,
+                                 ScriptingServer::scriptingPrintParameters,
+                                 NULL, NULL, NULL,
+                                 (void*)&settings,
+                                 reregInterval, runtimeLimit, quiet,
+                                 (struct TagItem*)&tags);
+   }
 
    rsp_freeinfo(&info);
    return(0);
