@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: standardservices.cc 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: standardservices.cc 1391 2007-05-23 07:27:45Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "standardservices.h"
@@ -61,7 +65,7 @@ EventHandlingResult EchoServer::handleMessage(rserpool_session_t sessionID,
    sent = rsp_sendmsg(RSerPoolSocketDescriptor,
                       buffer, bufferSize, 0,
                       sessionID, ppid, streamID,
-                      0, 0);
+                      0, 0, 0);
    return((sent == (ssize_t)bufferSize) ? EHR_Okay : EHR_Abort);
 }
 
@@ -134,10 +138,15 @@ void DaytimeServer::handleNotification(const union rserpool_notification* notifi
 
       ssize_t sent;
       sent = rsp_sendmsg(RSerPoolSocketDescriptor,
-                         (char*)&daytime, strlen(daytime), SCTP_EOF,
+                         (char*)&daytime, strlen(daytime), 0,
                          notification->rn_session_change.rsc_session,
                          0x00000000, 0,
-                         0, 0);
+                         0, 0, 0);
+      rsp_sendmsg(RSerPoolSocketDescriptor,
+                  NULL, 0, 0,
+                  notification->rn_session_change.rsc_session,
+                  0x00000000, 0,
+                  0, SCTP_EOF, 0);
    }
 }
 
@@ -169,12 +178,12 @@ EventHandlingResult CharGenServer::initializeSession()
    size_t        i;
 
    for(i = 0;i < sizeof(buffer);i++) {
-      buffer[i] = (unsigned char)(30 + (i % (256 - 30)));
+      buffer[i] = (unsigned char)(30 + (i % (128 - 30)));
    }
    while(rsp_sendmsg(RSerPoolSocketDescriptor,
                      (char*)&buffer, sizeof(buffer), 0,
-                     0, 0, 0, 0, 3600000000ULL) > 0) {
-      puts("sent data!");
+                     0, 0, 0, 0, 0, 3600000000ULL) > 0) {
+      // puts("sent data!");
    }
 
    return(EHR_Shutdown);
@@ -263,7 +272,7 @@ EventHandlingResult PingPongServer::handleMessage(const char* buffer,
 
             sent = rsp_sendmsg(RSerPoolSocketDescriptor,
                                (char*)pong, sizeof(Pong) + dataLength, 0,
-                               0, htonl(PPID_PPP), 0, 0, 0);
+                               0, htonl(PPID_PPP), 0, 0, 0, 0);
             if(sent > 0) {
                ReplyNo++;
                Replies++;
