@@ -1,31 +1,36 @@
-/*
- * IPv4 Subnet Calculator
- * Copyright (C) 2005-2007 by Thomas Dreibholz
+/* $Id: subnetcalc.cc 1726 2008-02-14 09:56:55Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: subnetcalc.cc 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
+ *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
+#include "tdtypes.h"
+
 #include <stdio.h>
-#include <unistd.h>
 #include <stdlib.h>
-#include <stdint.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
