@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: takeoverprocess.c 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: takeoverprocess.c 1284 2007-01-10 18:41:25Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "takeoverprocess.h"
@@ -40,13 +45,13 @@ struct TakeoverProcess* takeoverProcessNew(
    takeoverProcess = (struct TakeoverProcess*)malloc(sizeof(struct TakeoverProcess) +
                                                      sizeof(RegistrarIdentifierType) * peers);
    if(takeoverProcess != NULL) {
-      takeoverProcess->OutstandingAcknowledgements = 0;
+      takeoverProcess->OutstandingAcks = 0;
       peerListNode = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(&peerList->List);
       while(peerListNode != NULL) {
          if((peerListNode->Identifier != targetID) &&
             (peerListNode->Identifier != peerList->List.OwnIdentifier) &&
             (peerListNode->Identifier != 0)) {
-            takeoverProcess->PeerIDArray[takeoverProcess->OutstandingAcknowledgements++] =
+            takeoverProcess->PeerIDArray[takeoverProcess->OutstandingAcks++] =
                peerListNode->Identifier;
          }
          peerListNode = ST_CLASS(peerListGetNextPeerListNodeFromIndexStorage)(&peerList->List, peerListNode);
@@ -64,6 +69,13 @@ void takeoverProcessDelete(struct TakeoverProcess* takeoverProcess)
 }
 
 
+/* ###### Get number of outstanding acknowledgements ##################### */
+size_t takeoverProcessGetOutstandingAcks(const struct TakeoverProcess* takeoverProcess)
+{
+   return(takeoverProcess->OutstandingAcks);
+}
+
+
 /* ###### Acknowledge takeover process ################################### */
 size_t takeoverProcessAcknowledge(struct TakeoverProcess*       takeoverProcess,
                                   const RegistrarIdentifierType targetID,
@@ -71,13 +83,13 @@ size_t takeoverProcessAcknowledge(struct TakeoverProcess*       takeoverProcess,
 {
    size_t i;
 
-   for(i = 0;i < takeoverProcess->OutstandingAcknowledgements;i++) {
+   for(i = 0;i < takeoverProcess->OutstandingAcks;i++) {
       if(takeoverProcess->PeerIDArray[i] == acknowledgerID) {
-         for(   ;i < takeoverProcess->OutstandingAcknowledgements - 1;i++) {
+         for(   ;i < takeoverProcess->OutstandingAcks - 1;i++) {
             takeoverProcess->PeerIDArray[i] = takeoverProcess->PeerIDArray[i + 1];
          }
-         takeoverProcess->OutstandingAcknowledgements--;
+         takeoverProcess->OutstandingAcks--;
       }
    }
-   return(takeoverProcess->OutstandingAcknowledgements);
+   return(takeoverProcess->OutstandingAcks);
 }
