@@ -1,35 +1,42 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: testregistrator.c 1742 2008-02-14 17:15:25Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: testregistrator.c 1288 2007-01-11 20:07:41Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "rserpool.h"
 #include "breakdetector.h"
 #ifdef ENABLE_CSP
 #include "componentstatusreporter.h"
-#include "randomizer.h"
 #endif
 
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
+
 
 #define MAX_POOL_ELEMENTS 50000
 
@@ -41,10 +48,9 @@ int main(int argc, char** argv)
    struct rsp_loadinfo loadinfo;
    int           poolElementArray[MAX_POOL_ELEMENTS];
    char          myPoolHandle[512];
-   char*         poolHandle       = "TestPool";
-   size_t        elements         = 10;
+   const char*   poolHandle       = "TestPool";
+   size_t        poolElements     = 10;
    size_t        pools            = 0;
-   size_t        poolSize         = 10;
    bool          fastBreak        = false;
    bool          noDeregistration = false;
    size_t        i;
@@ -69,9 +75,6 @@ int main(int argc, char** argv)
       else if(!(strcmp(argv[i], "-noderegistration"))) {
          noDeregistration = true;
       }
-      else if(!(strncmp(argv[i], "-ph=" ,4))) {
-         poolHandle = (char*)&argv[i][4];
-      }
       else if(!(strncmp(argv[i], "-load=" ,6))) {
          loadinfo.rli_load = atol((char*)&argv[i][6]);
          if(loadinfo.rli_load > PPV_MAX_LOAD) {
@@ -90,19 +93,19 @@ int main(int argc, char** argv)
             loadinfo.rli_weight = 1;
          }
       }
-      else if(!(strncmp(argv[i], "-elements=" ,10))) {
-         elements = atol((char*)&argv[i][10]);
-         if(elements < 1) {
-            elements = 1;
+      else if(!(strncmp(argv[i], "-poolelements=" ,14))) {
+         poolElements = atol((char*)&argv[i][14]);
+         if(poolElements < 1) {
+            poolElements = 1;
          }
-         else if(elements > MAX_POOL_ELEMENTS) {
-            elements = MAX_POOL_ELEMENTS;
+         else if(poolElements > MAX_POOL_ELEMENTS) {
+            poolElements = MAX_POOL_ELEMENTS;
          }
       }
-      else if(!(strncmp(argv[i], "-poolsize=" ,10))) {
-         poolSize = atol((char*)&argv[i][10]);
-         if(poolSize < 1) {
-            poolSize = 1;
+      else if(!(strncmp(argv[i], "-pools=" ,7))) {
+         pools = atol((char*)&argv[i][7]);
+         if(pools < 1) {
+            pools = 1;
          }
       }
       else if(!(strncmp(argv[i], "-policy=" ,8))) {
@@ -149,7 +152,7 @@ int main(int argc, char** argv)
       }
       else {
          fprintf(stderr, "Bad argument \"%s\"!\n" ,argv[i]);
-         fprintf(stderr, "Usage: %s {-elements=total PEs} {-poolsize=PEs} {-poolhandle=pool handle} {-fastbreak} {-noderegistration} {-logfile=file|-logappend=file|-logquiet} {-loglevel=level} {-logcolor=on|off} {-policy=roundrobin|rr|weightedroundrobin|wee|leastused|lu|leastuseddegradation|lud|random|rd|weightedrandom|wrd} {-load=load} {-weight=weight} \n" ,
+         fprintf(stderr, "Usage: %s {-poolelements=total PEs} {-pools=PEs} {-poolhandle=pool handle} {-fastbreak} {-noderegistration} {-logfile=file|-logappend=file|-logquiet} {-loglevel=level} {-logcolor=on|off} {-policy=roundrobin|rr|weightedroundrobin|wee|leastused|lu|leastuseddegradation|lud|random|rd|weightedrandom|wrd} {-load=load} {-weight=weight} \n" ,
                 argv[0]);
          exit(1);
       }
@@ -163,24 +166,22 @@ int main(int argc, char** argv)
 
    puts("Test-Registrator - Version 1.0");
    puts("==============================\n");
-   printf("No Deregistration   = %s\n", (noDeregistration == true) ? "on" : "off");
-   printf("Fast Break          = %s\n", (fastBreak == true) ? "on" : "off");
-   printf("Total Pool Elements = %u\n", (unsigned int)elements);
-   printf("Pool Size           = %u PEs\n\n", (unsigned int)poolSize);
+   printf("No Deregistration = %s\n", (noDeregistration == true) ? "on" : "off");
+   printf("Fast Break        = %s\n", (fastBreak == true) ? "on" : "off");
+   printf("Pool              = %u PEs\n\n", (unsigned int)pools);
+   printf("Pool Elements     = %u\n", (unsigned int)poolElements);
 
 
-   for(i = 0;i < elements;i++) {
-      printf("Registering PE #%u...\n", (unsigned int)i + 1);
-      if(((i % poolSize) == 0) || (i == 0)) {
-         snprintf((char*)&myPoolHandle, sizeof(myPoolHandle), "%s-%04u", poolHandle, (unsigned int)++pools);
-      }
+   for(i = 0;i < poolElements;i++) {
+      snprintf((char*)&myPoolHandle, sizeof(myPoolHandle), "%s-%04u", poolHandle, (unsigned int)(1 + (i % pools)));
+      printf("Registering PE #%u in pool %s...\n", (unsigned int)i + 1, myPoolHandle);
       poolElementArray[i] = rsp_socket(0, SOCK_SEQPACKET, IPPROTO_SCTP);
       if(poolElementArray[i] < 0) {
          perror("Unable to create RSerPool socket");
          exit(1);
       }
       if(rsp_register(poolElementArray[i],
-                      (const unsigned char*)poolHandle, strlen(poolHandle),
+                      (const unsigned char*)myPoolHandle, strlen(myPoolHandle),
                       &loadinfo, 60000, 0) != 0) {
          perror("Unable to register pool element");
          exit(1);
@@ -196,7 +197,7 @@ int main(int argc, char** argv)
    }
 
    if(!noDeregistration) {
-      for(i = 0;i < elements;i++) {
+      for(i = 0;i < poolElements;i++) {
          rsp_deregister(poolElementArray[i], 0);
          rsp_close(poolElementArray[i]);
       }
