@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: udplikeserver.cc 1726 2008-02-14 09:56:55Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: udplikeserver.cc 1391 2007-05-23 07:27:45Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "udplikeserver.h"
@@ -29,6 +33,9 @@
 #include "breakdetector.h"
 #include "tagitem.h"
 
+#include <math.h>
+#include <string.h>
+
 
 // ###### Constructor #######################################################
 UDPLikeServer::UDPLikeServer()
@@ -137,13 +144,16 @@ void UDPLikeServer::printParameters()
 
 
 // ###### Wait for action on RSerPool socket ################################
-void UDPLikeServer::waitForAction(unsigned long long timeout)
+bool UDPLikeServer::waitForAction(unsigned long long timeout)
 {
    struct pollfd pollfds[1];
    pollfds[0].fd      = RSerPoolSocketDescriptor;
    pollfds[0].events  = POLLIN;
    pollfds[0].revents = 0;
-   rsp_poll((struct pollfd*)&pollfds, 1, (int)(timeout / 1000));
+   if(rsp_poll((struct pollfd*)&pollfds, 1, (int)(timeout / 1000)) > 0) {
+      return(pollfds[0].revents & POLLIN);
+   }
+   return(false);
 }
 
 
@@ -154,6 +164,7 @@ void UDPLikeServer::poolElement(const char*          programTitle,
                                 struct rsp_loadinfo* loadinfo,
                                 unsigned int         reregInterval,
                                 unsigned int         runtimeLimit,
+                                const bool           quiet,
                                 struct TagItem*      tags)
 {
    if(rsp_initialize(info) < 0) {
@@ -170,40 +181,45 @@ void UDPLikeServer::poolElement(const char*          programTitle,
          loadinfo = &dummyLoadinfo;
          loadinfo->rli_policy = PPT_ROUNDROBIN;
       }
+      loadinfo->rli_load = (unsigned int)rint(getLoad() * (double)PPV_MAX_LOAD);
       Load = loadinfo->rli_load;
 
       // ====== Print program title =========================================
-      puts(programTitle);
-      for(size_t i = 0;i < strlen(programTitle);i++) {
-         printf("=");
+      if(!quiet) {
+         puts(programTitle);
+         for(size_t i = 0;i < strlen(programTitle);i++) {
+            printf("=");
+         }
+         const char* policyName = rsp_getpolicybytype(loadinfo->rli_policy);
+         puts("\n\nGeneral Parameters:");
+         printf("   Pool Handle             = %s\n", poolHandle);
+         printf("   Reregistration Interval = %u [ms]\n", reregInterval);
+         printf("   Runtime Limit           = ");
+         if(runtimeLimit > 0) {
+            printf("%u [s]\n", runtimeLimit);
+         }
+         else {
+            puts("off");
+         }
+         puts("   Policy Settings");
+         printf("      Policy Type          = %s\n", (policyName != NULL) ? policyName : "?");
+         printf("      Load Degradation     = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
+         printf("      Load DPF             = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
+         printf("      Weight               = %u\n", loadinfo->rli_weight);
+         printf("      Weight DPF           = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
+         printParameters();
       }
-      const char* policyName = rsp_getpolicybytype(loadinfo->rli_policy);
-      puts("\n\nGeneral Parameters:");
-      printf("   Pool Handle             = %s\n", poolHandle);
-      printf("   Reregistration Interval = %u [ms]\n", reregInterval);
-      printf("   Runtime Limit           = ");
-      if(runtimeLimit > 0) {
-         printf("%u [s]\n", runtimeLimit);
-      }
-      else {
-         puts("off");
-      }
-      puts("   Policy Settings");
-      printf("      Policy Type          = %s\n", (policyName != NULL) ? policyName : "?");
-      printf("      Load Degradation     = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
-      printf("      Load DPF             = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
-      printf("      Weight               = %u\n", loadinfo->rli_weight);
-      printf("      Weight DPF           = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
-      printParameters();
 
       // ====== Register PE =================================================
       if(rsp_register_tags(RSerPoolSocketDescriptor,
                            (const unsigned char*)poolHandle, strlen(poolHandle),
                            loadinfo, reregInterval, 0, tags) == 0) {
-         uint32_t identifier;
-         if(rsp_getsockname(RSerPoolSocketDescriptor, NULL, NULL, &identifier) == 0) {
-            puts("Registration");
-            printf("   Identifier              = $%08x\n\n", identifier);
+         if(!quiet) {
+            uint32_t identifier;
+            if(rsp_getsockname(RSerPoolSocketDescriptor, NULL, NULL, &identifier) == 0) {
+               puts("Registration");
+               printf("   Identifier              = $%08x\n\n", identifier);
+            }
          }
          double oldLoad = (unsigned int)rint((double)loadinfo->rli_load / (double)PPV_MAX_LOAD);
 
@@ -216,6 +232,7 @@ void UDPLikeServer::poolElement(const char*          programTitle,
                (runtimeLimit > 0) ? (getMicroTime() + (1000ULL * runtimeLimit)) : 0;
             installBreakDetector();
             while(!breakDetected()) {
+
                // ====== Read from socket ===================================
                char                     buffer[65536];
                int                      flags = 0;
@@ -232,43 +249,45 @@ void UDPLikeServer::poolElement(const char*          programTitle,
                }
 
                // ====== Wait for action on RSerPool socket =================
-               waitForAction(timeout);
+               if(waitForAction(timeout)) {
 
-               // ====== Read from socket ===================================
-               ssize_t received = rsp_recvfullmsg(RSerPoolSocketDescriptor,
-                                                  (char*)&buffer, sizeof(buffer),
-                                                  &rinfo, &flags, 0);
+                  // ====== Read from socket ================================
+                  ssize_t received = rsp_recvfullmsg(RSerPoolSocketDescriptor,
+                                                     (char*)&buffer, sizeof(buffer),
+                                                     &rinfo, &flags, 0);
 
-               // ====== Handle data ========================================
-               if(received > 0) {
-                  // ====== Handle event ====================================
-                  EventHandlingResult eventHandlingResult;
-                  if(flags & MSG_RSERPOOL_NOTIFICATION) {
-                     handleNotification((const union rserpool_notification*)&buffer);
-                     /*
-                        We cannot shutdown or abort, since the session ID is not
-                        inserted into rinfo!
-                        Should we dissect the notification for the ID here?
-                     */
-                     eventHandlingResult = EHR_Okay;
-                  }
-                  else if(flags & MSG_RSERPOOL_COOKIE_ECHO) {
-                     eventHandlingResult = handleCookieEcho(rinfo.rinfo_session,
-                                                            (char*)&buffer, received);
-                  }
-                  else {
-                     eventHandlingResult = handleMessage(rinfo.rinfo_session,
-                                                         (char*)&buffer, received,
-                                                         rinfo.rinfo_ppid,
-                                                         rinfo.rinfo_stream);
-                  }
+                  // ====== Handle data =====================================
+                  if(received > 0) {
+                     // ====== Handle event =================================
+                     EventHandlingResult eventHandlingResult;
+                     if(flags & MSG_RSERPOOL_NOTIFICATION) {
+                        handleNotification((const union rserpool_notification*)&buffer);
+                        /*
+                           We cannot shutdown or abort, since the session ID is not
+                           inserted into rinfo!
+                           Should we dissect the notification for the ID here?
+                        */
+                        eventHandlingResult = EHR_Okay;
+                     }
+                     else if(flags & MSG_RSERPOOL_COOKIE_ECHO) {
+                        eventHandlingResult = handleCookieEcho(rinfo.rinfo_session,
+                                                               (char*)&buffer, received);
+                     }
+                     else {
+                        eventHandlingResult = handleMessage(rinfo.rinfo_session,
+                                                            (char*)&buffer, received,
+                                                            rinfo.rinfo_ppid,
+                                                            rinfo.rinfo_stream);
+                     }
 
-                  // ====== Check for problems ==============================
-                  if((eventHandlingResult == EHR_Abort) ||
-                     (eventHandlingResult == EHR_Shutdown)) {
-                     rsp_sendmsg(RSerPoolSocketDescriptor,
-                                 NULL, 0, ((eventHandlingResult == EHR_Abort) ? SCTP_ABORT : SCTP_EOF),
-                                 rinfo.rinfo_session, 0, 0, 0, 0);
+                     // ====== Check for problems ===========================
+                     if((eventHandlingResult == EHR_Abort) ||
+                        (eventHandlingResult == EHR_Shutdown)) {
+                        rsp_sendmsg(RSerPoolSocketDescriptor,
+                                    NULL, 0, 0,
+                                    rinfo.rinfo_session, 0, 0, 0,
+                                    ((eventHandlingResult == EHR_Abort) ? SCTP_ABORT : SCTP_EOF), 0);
+                     }
                   }
                }
 
