@@ -1,26 +1,30 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: udplikeserver.h 1667 2008-02-04 10:12:05Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: udplikeserver.h 1391 2007-05-23 07:27:45Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2008 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef UDPLIKESERVER_H
@@ -47,13 +51,14 @@ class UDPLikeServer
                             struct rsp_loadinfo* loadinfo,
                             unsigned int         reregInterval = 30000,
                             unsigned int         runtimeLimit  = 0,
+                            const bool           quiet         = false,
                             struct TagItem*      tags          = NULL);
 
    double getLoad() const;
    void setLoad(double load);
 
    protected:
-   virtual void waitForAction(unsigned long long timeout);
+   virtual bool waitForAction(unsigned long long timeout);
    virtual void printParameters();
    void startTimer(unsigned long long timeStamp);
    void stopTimer();
